#ifndef __EXCEPTION_H__
#define __EXCEPTION_H__

#include <new>
#include <cstring>

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

// Create a macro to allow extended storage specification under a Visual C++ build
#ifdef _MSC_VER
	#define MSC_EX_DECL(x) __declspec(x)
#else
	#define MSC_EX_DECL(x)
#endif

namespace StdExtensions
{
	const size_t EXCEPTION_BUFFSIZE = 255;

	// Enumeration of error codes - C standard requires that EDOM, EILSEQ and ERANGE be defined
	enum ErrorClass
	{
		e_UnknownError,			// Error is not a standard type
		e_HeapMemoryLow,		// Out of heap memory
		e_DomainError,			// Arguments are out of bounds
		e_RangeError,			// Unable to output result without losing accuracy or precision
		e_CastError				// Unable to cast one pointer as another with dynamic_cast
	};

	// --------------------------
	//  CLASS FOR ERROR HANDLING
	// --------------------------
	// Class for holding error messages, including a desciption of the error and the location
	// in the source code from where the error was thrown
	class Exception
	{
	private:
		size_t ulLine;							// Line from which the error was thrown
		char pRoutine[EXCEPTION_BUFFSIZE + 1];	// Routine from which the error was thrown
		char pFile[EXCEPTION_BUFFSIZE + 1];		// File from where the error was thrown
		char pMessage[EXCEPTION_BUFFSIZE + 1];	// Error message
		ErrorClass cErrorType;					// Error type
	public:
		// Empty initialisation functions
		Exception() throw() {Initialise();};
		void Initialise() throw();
		// Parameterised initialisation functions
		Exception(const char *pMessageIn, const char *pRoutineIn, size_t ulLineIn, const char *pFileIn, ErrorClass cTypeIn = e_UnknownError) throw()
			{Initialise(pMessageIn, pRoutineIn, ulLineIn, pFileIn, cTypeIn);};
		void Initialise(const char *pMessageIn, const char *pRoutineIn, size_t ulLineIn, const char *pFileIn, ErrorClass cTypeIn = e_UnknownError) throw();
		// Copy initialisation functions
		Exception(const Exception &cObIn) throw() {Initialise(cObIn);};
		void Initialise(const Exception &cObIn) throw();
		Exception& operator = (const Exception &cObIn) throw() {Initialise(cObIn); return *this;};
		// Destructor function
		~Exception() throw() {Initialise();};
		// Getter error message details
		char *GetErrorMessage(char *pStrOut, size_t ulSizeOut) const throw();					// Get error message
		char *GetErrorRoutine(char *pStrOut, size_t ulSizeOut) const throw();					// Get error routine
		char *GetErrorFile(char *pStrOut, size_t ulSizeOut) const throw();						// Get error file
		ErrorClass GetErrorClass() const throw() {return cErrorType;};							// Get error code
		size_t GetErrorLine() const throw() {return ulLine;};									// Get eror line
		// Tests of equality
		bool Equal(const Exception &cObIn) const throw();
		bool NotEqual(const Exception &cObIn) const throw() {return Equal(cObIn) ? false : true;};
		bool operator == (const Exception &cObIn) const throw() {return Equal(cObIn);};
		bool operator != (const Exception &cObIn) const throw() {return NotEqual(cObIn);};
	};

	// --------------------------------
	//  MAIN EXCEPTION RAISER FUNCTION
	// --------------------------------

	// Raise an exception
	void RaiseException(const char *pMessage, size_t ulLine, const char *pRoutine, const char *pFile, ErrorClass cErrorType = e_UnknownError) throw(Exception);

	// -----------------------------
	//  MEMORY ALLOCATION FUNCTIONS
	// -----------------------------

	// Allocate memory function - handle relevant exception schema
	template<class DTypeA> DTypeA *AllocateMemory() throw(Exception)
	{
		DTypeA *pOut = NULL;
		try
		{	// Allocate memory
			pOut = new DTypeA;
			if(pOut == NULL)
			{	// Check for null allocation as some compilers do not support std::bad_alloc
				RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory()", __FILE__, e_HeapMemoryLow);
			}
		}
		catch(std::bad_alloc)
		{	// Allocation exception - rethrow as standard exception
			pOut = NULL;
			RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory()", __FILE__, e_HeapMemoryLow);
		}
		catch(Exception)
		{	// Standard exception thrown - rethrow
			pOut = NULL;
			throw;
		}
		catch(...)
		{	// Non-standard exception thrown - rethrow as standard exception
			pOut = NULL;
			RaiseException("unknown exception thrown during heap allocation", __LINE__, "StdExtensions::AllocateMemory()", __FILE__, e_UnknownError);
		}
		return pOut;
	}

	// Allocate memory function - handle relevant exception schema
	template<class DTypeA> DTypeA *AllocateMemory(size_t ulSize) throw(Exception)
	{
		DTypeA *pOut = NULL;
		try
		{	// Allocate memory
			if(ulSize <= 0)
			{
				RaiseException("unable to allocate memory for an array of zero length", __LINE__, "StdExtensions::AllocateMemory(size_t)", __FILE__, e_DomainError);
			}
			else
			{	// Allocate memory
				pOut = new DTypeA [ulSize];
				if(pOut == NULL)
				{	// Check for null allocation as some compilers do not support std::bad_alloc
					RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory(size_t)", __FILE__, e_HeapMemoryLow);
				}
			}
		}
		catch(std::bad_alloc)
		{	// Allocation exception - rethrow as standard exception
			pOut = NULL;
			RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory(size_t)", __FILE__, e_HeapMemoryLow);
		}
		catch(Exception)
		{	// Standard exception throw - rethrow
			pOut = NULL;
			throw;
		}
		catch(...)
		{	// Non-standard exception thrown - rethrow as standard exception
			pOut = NULL;
			RaiseException("unknown exception thrown during heap allocation", __LINE__, "StdExtensions::AllocateMemory(size_t)", __FILE__, e_UnknownError);
		}
		return pOut;
	}

	// Function to allocate memory for the copy of an object - handle relevant exception schema
	template<class DTypeA> DTypeA *AllocateMemory(const DTypeA &cObIn) throw(Exception)
	{
		DTypeA *pOut = NULL;
		try
		{	// Allocate memory
			pOut = new DTypeA(cObIn);
			if(pOut == NULL)
			{	// Check for null allocation as some compilers do not support std::bad_alloc
				RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory(const DTypeA &)", __FILE__, e_HeapMemoryLow);
			}
		}
		catch(std::bad_alloc)
		{	// Allocation exception - rethrow as standard exception
			pOut = NULL;
			RaiseException("out of memory", __LINE__, "StdExtensions::AllocateMemory(const DTypeA &)", __FILE__, e_HeapMemoryLow);
		}
		catch(Exception)
		{	// Standard exception thrown - rethrow
			pOut = NULL;
			throw;
		}
		catch(...)
		{	// Non-standard exception thrown - rethrow as standard exception
			pOut = NULL;
			RaiseException("unknown exception thrown during heap allocation", __LINE__, "StdExtensions::AllocateMemory(const DTypeA &)", __FILE__, e_UnknownError);
		}
		return pOut;
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
