#ifndef __EXTNUMBER_H__
#define __EXTNUMBER_H__

#include "MathFuncs.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace StdExtensions
{	// --------------------
	//  SPECIAL VALUE ENUM
	// --------------------
	// Enumeration to hold special values
	enum SpecialValues
	{
		e_Standard,				// Number does not have a special value
		e_MissingValue,			// Number is a missing value
		e_NotANumber,			// Number is mathematically undefined
		e_PositiveInfinity,		// Number is positive infinity
		e_NegativeInfinity		// Number is negative infinity
	};

	// Values for the extended boolean class
	const int EXTBOOL_FALSE = 0;
	const int EXTBOOL_TRUE = 1;
	const int EXTBOOL_NA = 2;

	// ------------------------
	//  EXTENDED BOOLEAN CLASS
	// ------------------------
	// Extension of the boolean class that also incorporates information on missing values
	class ExtBool
	{
	private:
		int iBool;				// Boolean value EXTBOOL_FALSE = FALSE, EXTBOOL_TRUE = TRUE, and EXTBOOL_NA = NA
	public:
		// Empty initialisation function
		ExtBool() throw() : iBool(EXTBOOL_NA) {};
		void Initialise() throw() {iBool = EXTBOOL_NA;};
		// Initialisation using a boolean value
		ExtBool(bool bBool) throw() : iBool(bBool ? EXTBOOL_TRUE : EXTBOOL_FALSE) {};
		void Initialise(bool bBool) throw() {iBool = bBool ? EXTBOOL_TRUE : EXTBOOL_FALSE;};
		ExtBool& operator = (bool bBool) throw() {Initialise(bBool); return *this;};
		// Initialisation using another extended boolean type
		ExtBool(const ExtBool &cObIn) throw() : iBool(cObIn.iBool) {};
		void Initialise(const ExtBool &cObIn) throw() {iBool = cObIn.iBool;};
		ExtBool& operator = (const ExtBool &cObIn) throw() {Initialise(cObIn); return *this;};
		// Initialisation using a missing data value
		ExtBool(SpecialValues eSpecIn) throw(Exception) : iBool(EXTBOOL_NA) {Initialise(eSpecIn);};
		void Initialise(SpecialValues eSpecIn) throw(Exception);
		ExtBool& operator = (SpecialValues eSpecIn) throw(Exception) {Initialise(eSpecIn); return *this;};
		// Get as a native boolean type
		bool GetAsBool() const throw(Exception);
		operator bool() const throw(Exception) {return GetAsBool();};
		// Inverse the true/false value
		ExtBool Inverse() const throw();
		ExtBool operator ! () const throw() {return Inverse();};
		// Comparison functions
		ExtBool Equal(const ExtBool &cObIn) const throw();
		ExtBool Equal(bool bBool) const throw() {return Equal(ExtBool(bBool));};
		ExtBool NotEqual(const ExtBool &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool NotEqual(bool bBool) const throw() {return Equal(bBool).Inverse();};
		// Logical functions
		//		&&
		ExtBool And(const ExtBool &cObIn) const throw();
		ExtBool And(bool bBool) const throw() {return And(ExtBool(bBool));};
		//		||
		ExtBool Or(const ExtBool &cObIn) const throw();
		ExtBool Or(bool bBool) const throw() {return Or(ExtBool(bBool));};
		//		XOR
		ExtBool XOr(const ExtBool &cObIn) const throw();
		ExtBool XOr(bool bBool) const throw() {return XOr(ExtBool(bBool));};
		// Overloaded comparison operators
		ExtBool operator == (const ExtBool &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator == (bool bBool) const throw() {return Equal(bBool);};
		ExtBool operator != (const ExtBool &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool operator != (bool bBool) const throw() {return NotEqual(bBool);};
		// Overloaded logical operators
		ExtBool operator && (const ExtBool &cObIn) const throw() {return And(cObIn);};
		ExtBool operator && (bool bBool) const throw() {return And(bBool);};
		ExtBool operator || (const ExtBool &cObIn) const throw() {return Or(cObIn);};
		ExtBool operator || (bool bBool) const throw() {return Or(bBool);};
		// Returns true if extended boolean is a missing value
		bool IsMissingValue() const throw() {return iBool == EXTBOOL_NA ? true : false;};
	};

	// Overloaded operators for interaction between native boolean and extended boolean types
	inline ExtBool operator == (bool bBool, const ExtBool &cObIn) throw() {return cObIn.Equal(bBool);}
	inline ExtBool operator != (bool bBool, const ExtBool &cObIn) throw() {return cObIn.NotEqual(bBool);}
	inline ExtBool operator && (bool bBool, const ExtBool &cObIn) throw() {return cObIn.And(bBool);}
	inline ExtBool operator || (bool bBool, const ExtBool &cObIn) throw() {return cObIn.Or(bBool);}

	// -----------------------
	//  EXTENDED DOUBLE CLASS
	// -----------------------
	// Extension of the double number class that also incorporates information on missing values,
	// numbers which are not mathematically defined and concepts of infinity.
	class ExtDouble
	{
	private:
		double dVal;			// Base value of the double
		SpecialValues eSpec;	// Extension to carry possible special values
	public:
		// Empty initialisation functions
		ExtDouble() throw() : dVal(0.0), eSpec(e_MissingValue) {};
		void Initialise() throw() {dVal = 0.0; eSpec = e_MissingValue;};
		// Parameterised initialisation functions - normal values
		ExtDouble(double dIn) throw() : dVal(dIn), eSpec(e_Standard) {};
		void Initialise(double dIn) throw() {dVal = dIn; eSpec = e_Standard;};
		ExtDouble& operator = (double dIn) throw() {Initialise(dIn); return *this;};
		// Parameterised initialisation functions - special values
		ExtDouble(SpecialValues eSpecIn) throw() : dVal(0.0), eSpec(eSpecIn) {};
		void Initialise(SpecialValues eSpecIn) throw() {dVal = 0.0; eSpec = eSpecIn;};
		ExtDouble& operator = (SpecialValues eSpecIn) throw() {Initialise(eSpecIn); return *this;};
		// Copy initialisation functions
		ExtDouble(const ExtDouble &cObIn) throw() : dVal(cObIn.dVal), eSpec(cObIn.eSpec) {};
		void Initialise(const ExtDouble &cObIn) throw() {dVal = cObIn.dVal; eSpec = cObIn.eSpec;};
		ExtDouble& operator = (const ExtDouble &cObIn) throw() {Initialise(cObIn); return *this;};
		double GetAsDouble() const throw(Exception);									// Retrieve the object as a native double
		int GetAsInteger() const throw(Exception) {return int(GetAsDouble());};			// Retrieve the object as a native integer
		// Unary mathematical functions
		ExtDouble Negative() const throw();		// Returns the negative of the calling object
		ExtDouble Reciprocal() const throw();	// Returns the reciprocal of the calling object
		// Arithmetic functions
		//		addition
		ExtDouble Add(const ExtDouble &cObIn) const throw();
		ExtDouble Add(double dIn) const throw() {return Add(ExtDouble(dIn));};
		ExtDouble Add(SpecialValues eSpecIn) const throw() {return Add(ExtDouble(eSpecIn));};
		//		subtraction
		ExtDouble Subtract(const ExtDouble &cObIn) const throw() {return Add(cObIn.Negative());};
		ExtDouble Subtract(double dIn) const throw() {return Subtract(ExtDouble(dIn));};
		ExtDouble Subtract(SpecialValues eSpecIn) const throw() {return Subtract(ExtDouble(eSpecIn));};
		//		multiplication
		ExtDouble Multiply(const ExtDouble &cObIn) const throw();
		ExtDouble Multiply(double dIn) const throw() {return Multiply(ExtDouble(dIn));};
		ExtDouble Multiply(SpecialValues eSpecIn) const throw() {return Multiply(ExtDouble(eSpecIn));};
		//		division
		ExtDouble Divide(const ExtDouble &cObIn) const throw();
		ExtDouble Divide(double dIn) const throw() {return Divide(ExtDouble(dIn));};
		ExtDouble Divide(SpecialValues eSpecIn) const throw() {return Divide(ExtDouble(eSpecIn));};
		// Function to calculate the calling object and raised to a a power
		ExtDouble Pow(const ExtDouble &cObIn) const throw();
		ExtDouble Pow(double dIn) const throw() {return Pow(ExtDouble(dIn));};
		ExtDouble Pow(SpecialValues eSpecIn) const throw() {return Pow(ExtDouble(eSpecIn));};
		// Functions to test for special values
		bool IsMissingValue() const throw() {return eSpec == e_MissingValue ? true : false;};			// Test for missing values
		bool IsNotANumber() const throw() {return eSpec == e_NotANumber ? true : false;};				// Test for NaN values
		bool IsPositiveInfinity() const throw() {return eSpec == e_PositiveInfinity ? true : false;};	// Test for +Inf
		bool IsNegativeInfinity() const throw() {return eSpec == e_NegativeInfinity ? true : false;};	// Test for -Inf
		bool IsSpecialValue() const throw() {return eSpec == e_Standard ? false : true;};
		// Comparison functions
		//		equal
		ExtBool Equal(const ExtDouble &cObIn) const throw();
		ExtBool Equal(double dIn) const throw() {return Equal(ExtDouble(dIn));};
		ExtBool Equal(SpecialValues eSpecIn) const throw() {return Equal(ExtDouble(eSpecIn));};
		//		not equal
		ExtBool NotEqual(const ExtDouble &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool NotEqual(double dIn) const throw() {return NotEqual(ExtDouble(dIn));};
		ExtBool NotEqual(SpecialValues eSpecIn) const throw() {return NotEqual(ExtDouble(eSpecIn));};
		//		greater than
		ExtBool GreaterThan(const ExtDouble &cObIn) const throw();
		ExtBool GreaterThan(double dIn) const throw() {return GreaterThan(ExtDouble(dIn));};
		ExtBool GreaterThan(SpecialValues eSpecIn) const throw() {return GreaterThan(ExtDouble(eSpecIn));};
		//		greater than or equal
		ExtBool GreaterThanOrEqual(const ExtDouble &cObIn) const throw();
		ExtBool GreaterThanOrEqual(double dIn) const throw() {return GreaterThanOrEqual(ExtDouble(dIn));};
		ExtBool GreaterThanOrEqual(SpecialValues eSpecIn) const throw() {return GreaterThanOrEqual(ExtDouble(eSpecIn));};
		//		less than
		ExtBool LessThan(const ExtDouble &cObIn) const throw() {return cObIn.GreaterThan(*this);};
		ExtBool LessThan(double dIn) const throw() {return LessThan(ExtDouble(dIn));};
		ExtBool LessThan(SpecialValues eSpecIn) const throw() {return LessThan(ExtDouble(eSpecIn));};
		//		less than or equal
		ExtBool LessThanOrEqual(const ExtDouble &cObIn) const throw() {return cObIn.GreaterThanOrEqual(*this);};
		ExtBool LessThanOrEqual(double dIn) const throw() {return LessThanOrEqual(ExtDouble(dIn));};
		ExtBool LessThanOrEqual(SpecialValues eSpecIn) const throw() {return LessThanOrEqual(ExtDouble(eSpecIn));};
		// Overloaded comparison operators
		//		equality operators
		ExtBool operator == (const ExtDouble &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator == (double dIn) const throw() {return Equal(dIn);};
		ExtBool operator == (SpecialValues eSpecIn) const throw() {return Equal(eSpecIn);};
		//		inequality operators
		ExtBool operator != (const ExtDouble &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool operator != (double dIn) const throw() {return NotEqual(dIn);};
		ExtBool operator != (SpecialValues eSpecIn) const throw() {return NotEqual(eSpecIn);};
		//		greater than operators
		ExtBool operator > (const ExtDouble &cObIn) const throw() {return GreaterThan(cObIn);};
		ExtBool operator > (double dIn) const throw() {return GreaterThan(dIn);};
		ExtBool operator > (SpecialValues eSpecIn) const throw() {return GreaterThan(eSpecIn);};
		//		greater than or equal operators
		ExtBool operator >= (const ExtDouble &cObIn) const throw() {return GreaterThanOrEqual(cObIn);};
		ExtBool operator >= (double dIn) const throw() {return GreaterThanOrEqual(dIn);};
		ExtBool operator >= (SpecialValues eSpecIn) const throw() {return GreaterThanOrEqual(eSpecIn);};
		//		less than operators
		ExtBool operator < (const ExtDouble &cObIn) const throw() {return LessThan(cObIn);};
		ExtBool operator < (double dIn) const throw() {return LessThan(dIn);};
		ExtBool operator < (SpecialValues eSpecIn) const throw() {return LessThan(eSpecIn);};
		//		less than or equal operator
		ExtBool operator <= (const ExtDouble &cObIn) const throw() {return LessThanOrEqual(cObIn);};
		ExtBool operator <= (double dIn) const throw() {return LessThanOrEqual(dIn);};
		ExtBool operator <= (SpecialValues eSpecIn) const throw() {return LessThanOrEqual(eSpecIn);};
		// Overloaded arithmetic operators
		//		unary negation
		ExtDouble operator - () const throw() {return Negative();};
		//		increment operators
		ExtDouble& operator ++ () throw() {Initialise(Add(1.0)); return *this;};
		ExtDouble operator ++ (int) throw() {ExtDouble cOut; cOut.Initialise(*this); Initialise(Add(1.0)); return cOut;};
		//		decrement operators
		ExtDouble& operator -- () throw() {Initialise(Subtract(1.0)); return *this;};
		ExtDouble operator -- (int) throw() {ExtDouble cOut; cOut.Initialise(*this); Initialise(Subtract(1.0)); return cOut;};
		//		functional assignment operators
		//			addition
		ExtDouble& operator += (const ExtDouble &cObIn) throw() {Initialise(Add(cObIn)); return *this;};
		ExtDouble& operator += (double dIn) throw() {Initialise(Add(dIn)); return *this;};
		ExtDouble& operator += (SpecialValues eSpecIn) throw() {Initialise(Add(eSpecIn)); return *this;};
		//			subtraction
		ExtDouble& operator -= (const ExtDouble &cObIn) throw() {Initialise(Subtract(cObIn)); return *this;};
		ExtDouble& operator -= (double dIn) throw() {Initialise(Subtract(dIn)); return *this;};
		ExtDouble& operator -= (SpecialValues eSpecIn) throw() {Initialise(Subtract(eSpecIn)); return *this;};
		//			multiplication
		ExtDouble& operator *= (const ExtDouble &cObIn) throw() {Initialise(Multiply(cObIn)); return *this;};
		ExtDouble& operator *= (double dIn) throw() {Initialise(Multiply(dIn)); return *this;};
		ExtDouble& operator *= (SpecialValues eSpecIn) throw() {Initialise(Multiply(eSpecIn)); return *this;};
		//			division
		ExtDouble& operator /= (const ExtDouble &cObIn) throw() {Initialise(Divide(cObIn)); return *this;};
		ExtDouble& operator /= (double dIn) throw() {Initialise(Divide(dIn)); return *this;};
		ExtDouble& operator /= (SpecialValues eSpecIn) throw() {Initialise(Divide(eSpecIn)); return *this;};
		//		normal functional operators
		//			addition
		ExtDouble operator + (const ExtDouble &cObIn) const throw() {return Add(cObIn);};
		ExtDouble operator + (double dIn) const throw() {return Add(dIn);};
		ExtDouble operator + (SpecialValues eSpecIn) const throw() {return Add(eSpecIn);};
		//			subtraction
		ExtDouble operator - (const ExtDouble &cObIn) const throw() {return Subtract(cObIn);};
		ExtDouble operator - (double dIn) const throw() {return Subtract(dIn);};
		ExtDouble operator - (SpecialValues eSpecIn) const throw() {return Subtract(eSpecIn);};
		//			multiplication
		ExtDouble operator * (const ExtDouble &cObIn) const throw() {return Multiply(cObIn);};
		ExtDouble operator * (double dIn) const throw() {return Multiply(dIn);};
		ExtDouble operator * (SpecialValues eSpecIn) const throw() {return Multiply(eSpecIn);};
		//			division
		ExtDouble operator / (const ExtDouble &cObIn) const throw() {return Divide(cObIn);};
		ExtDouble operator / (double dIn) const throw() {return Divide(dIn);};
		ExtDouble operator / (SpecialValues eSpecIn) const throw() {return Divide(eSpecIn);};
	};

	// Overloaded operators for native double and extended double interaction
	//		normal arithmetic functions
	inline ExtDouble operator + (double dIn, const ExtDouble &cObIn) throw() {return cObIn.Add(dIn);}
	inline ExtDouble operator - (double dIn, const ExtDouble &cObIn) throw() {return cObIn.Negative().Add(dIn);}
	inline ExtDouble operator * (double dIn, const ExtDouble &cObIn) throw() {return cObIn.Multiply(dIn);}
	inline ExtDouble operator / (double dIn, const ExtDouble &cObIn) throw() {ExtDouble cTemp; cTemp.Initialise(dIn); return cTemp.Divide(cObIn);}
	inline ExtDouble operator + (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.Add(eSpecIn);}
	inline ExtDouble operator - (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.Negative().Add(eSpecIn);}
	inline ExtDouble operator * (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.Multiply(eSpecIn);}
	inline ExtDouble operator / (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {ExtDouble cTemp; cTemp.Initialise(eSpecIn); return cTemp.Divide(cObIn);}
	//		assignment arithmetic functions
	inline double& operator += (double &dIn, const ExtDouble &cObIn) throw(Exception) {return (dIn += cObIn.GetAsDouble());}
	inline double& operator -= (double &dIn, const ExtDouble &cObIn) throw(Exception) {return (dIn -= cObIn.GetAsDouble());}
	inline double& operator *= (double &dIn, const ExtDouble &cObIn) throw(Exception) {return (dIn *= cObIn.GetAsDouble());}
	inline double& operator /= (double &dIn, const ExtDouble &cObIn) throw(Exception) {return (dIn /= cObIn.GetAsDouble());}
	//		comparrison functions
	inline ExtBool operator == (double dIn, const ExtDouble &cObIn) throw() {return cObIn.Equal(dIn);}
	inline ExtBool operator != (double dIn, const ExtDouble &cObIn) throw() {return cObIn.NotEqual(dIn);}
	inline ExtBool operator > (double dIn, const ExtDouble &cObIn) throw() {return cObIn.LessThan(dIn);}
	inline ExtBool operator >= (double dIn, const ExtDouble &cObIn) throw() {return cObIn.LessThanOrEqual(dIn);}
	inline ExtBool operator < (double dIn, const ExtDouble &cObIn) throw() {return cObIn.GreaterThan(dIn);}
	inline ExtBool operator <= (double dIn, const ExtDouble &cObIn) throw() {return cObIn.GreaterThanOrEqual(dIn);}
	inline ExtBool operator == (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.Equal(eSpecIn);}
	inline ExtBool operator != (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.NotEqual(eSpecIn);}
	inline ExtBool operator > (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.LessThan(eSpecIn);}
	inline ExtBool operator >= (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.LessThanOrEqual(eSpecIn);}
	inline ExtBool operator < (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.GreaterThan(eSpecIn);}
	inline ExtBool operator <= (SpecialValues eSpecIn, const ExtDouble &cObIn) throw() {return cObIn.GreaterThanOrEqual(eSpecIn);}

	// ------------------------
	//  EXTENDED INTEGER CLASS
	// ------------------------
	// Extension of the integer number class that also incorporates informaiton on missing values,
	// numbers which are not mathematically defined and concepts of infinity.
	class ExtInteger
	{
	private:
		int iVal;				// Base value of the integer
		SpecialValues eSpec;	// Extension to carry possible special values
	public:
		// Empty initialisation functions
		ExtInteger() throw() : iVal(0), eSpec(e_MissingValue) {};
		void Initialise() throw() {iVal = 0; eSpec = e_MissingValue;};
		// Parameterised initialisation functions - normal values
		ExtInteger(int iIn) throw() : iVal(iIn), eSpec(e_Standard) {};
		void Initialise(int iIn) throw() {iVal = iIn; eSpec = e_Standard;};
		ExtInteger& operator = (int iIn) throw() {iVal = iIn; return *this;};
		// Parameterised initialisation functions - special values
		ExtInteger(SpecialValues eSpecIn) throw() : iVal(0), eSpec(eSpecIn) {};
		void Initialise(SpecialValues eSpecIn) throw() {iVal = 0; eSpec = eSpecIn;};
		ExtInteger& operator = (SpecialValues eSpecIn) throw() {Initialise(eSpecIn); return *this;};
		// Copy initialisation functions
		ExtInteger(const ExtInteger &cObIn) throw() : iVal(cObIn.iVal), eSpec(cObIn.eSpec) {};
		void Initialise(const ExtInteger &cObIn) throw() {iVal = cObIn.iVal; eSpec = cObIn.eSpec;};
		ExtInteger& operator = (const ExtInteger &cObIn) throw() {Initialise(cObIn); return *this;};
		int GetAsInteger() const throw(Exception);										// Retrieve the object as a native integer
		double GetAsDouble() const throw(Exception) {return double(GetAsInteger());};	// Retrieve the object as a native double
		// Unary mathematical functions
		ExtInteger Negative() const throw();	// Returns the negative of the calling object
		// Arithmetic functions
		//		addition
		ExtInteger Add(const ExtInteger &cObIn) const throw();
		ExtInteger Add(int iIn) const throw() {return Add(ExtInteger(iIn));};
		ExtInteger Add(SpecialValues eSpecIn) const throw() {return Add(ExtInteger(eSpecIn));};
		//		subtraction
		ExtInteger Subtract(const ExtInteger &cObIn) const throw() {return Add(cObIn.Negative());};
		ExtInteger Subtract(int iIn) const throw() {return Subtract(ExtInteger(iIn));};
		ExtInteger Subtract(SpecialValues eSpecIn) const throw() {return Subtract(ExtInteger(eSpecIn));};
		//		multiplication
		ExtInteger Multiply(const ExtInteger &cObIn) const throw();
		ExtInteger Multiply(int iIn) const throw() {return Multiply(ExtInteger(iIn));};
		ExtInteger Multiply(SpecialValues eSpecIn) const throw() {return Multiply(ExtInteger(eSpecIn));};
		//		division
		ExtInteger Divide(const ExtInteger &cObIn) const throw();
		ExtInteger Divide(int iIn) const throw() {return Divide(ExtInteger(iIn));};
		ExtInteger Divide(SpecialValues eSpecIn) const throw() {return Divide(ExtInteger(eSpecIn));};
		//		modulus
		ExtInteger Modulo(const ExtInteger &cObIn) const throw();
		ExtInteger Modulo(int iIn) const throw() {return Modulo(ExtInteger(iIn));};
		ExtInteger Modulo(SpecialValues eSpecIn) const throw() {return Modulo(ExtInteger(eSpecIn));};
		// Functions to test for special values
		bool IsMissingValue() const throw() {return eSpec == e_MissingValue ? true : false;};			// Test for missing values
		bool IsNotANumber() const throw() {return eSpec == e_NotANumber ? true : false;};				// Test for NaN values
		bool IsPositiveInfinity() const throw() {return eSpec == e_PositiveInfinity ? true : false;};	// Test for +Inf
		bool IsNegativeInfinity() const throw() {return eSpec == e_NegativeInfinity ? true : false;};	// Test for -Inf
		bool IsSpecialValue() const throw() {return eSpec == e_Standard ? false : true;};
		// Comparrison functions
		//		equal
		ExtBool Equal(const ExtInteger &cObIn) const throw();
		ExtBool Equal(int iIn) const throw() {return Equal(ExtInteger(iIn));};
		ExtBool Equal(SpecialValues eSpecIn) const throw() {return Equal(ExtInteger(eSpecIn));};
		//		not equal
		ExtBool NotEqual(const ExtInteger &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool NotEqual(int iIn) const throw() {return NotEqual(ExtInteger(iIn));};
		ExtBool NotEqual(SpecialValues eSpecIn) const throw() {return NotEqual(ExtInteger(eSpecIn));};
		//		greater than
		ExtBool GreaterThan(const ExtInteger &cObIn) const throw();
		ExtBool GreaterThan(int iIn) const throw() {return GreaterThan(ExtInteger(iIn));};
		ExtBool GreaterThan(SpecialValues eSpecIn) const throw() {return GreaterThan(ExtInteger(eSpecIn));};
		//		greater than or equal
		ExtBool GreaterThanOrEqual(const ExtInteger &cObIn) const throw();
		ExtBool GreaterThanOrEqual(int iIn) const throw() {return GreaterThanOrEqual(ExtInteger(iIn));};
		ExtBool GreaterThanOrEqual(SpecialValues eSpecIn) const throw() {return GreaterThanOrEqual(ExtInteger(eSpecIn));};
		//		less than
		ExtBool LessThan(const ExtInteger &cObIn) const throw() {return cObIn.GreaterThan(*this);};
		ExtBool LessThan(int iIn) const throw() {return LessThan(ExtInteger(iIn));};
		ExtBool LessThan(SpecialValues eSpecIn) const throw() {return LessThan(ExtInteger(eSpecIn));};
		//		less than or equal
		ExtBool LessThanOrEqual(const ExtInteger &cObIn) const throw() {return cObIn.GreaterThanOrEqual(*this);};
		ExtBool LessThanOrEqual(int iIn) const throw() {return LessThanOrEqual(ExtInteger(iIn));};
		ExtBool LessThanOrEqual(SpecialValues eSpecIn) const throw() {return LessThanOrEqual(ExtInteger(eSpecIn));};
		// Overloaded comparrison operators
		//		equality operators
		ExtBool operator == (const ExtInteger &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator == (int iIn) const throw() {return Equal(iIn);};
		ExtBool operator == (SpecialValues eSpecIn) const throw() {return Equal(eSpecIn);};
		//		inequality operators
		ExtBool operator != (const ExtInteger &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool operator != (int iIn) const throw() {return NotEqual(iIn);};
		ExtBool operator != (SpecialValues eSpecIn) const throw() {return NotEqual(eSpecIn);};
		//		greater than operators
		ExtBool operator > (const ExtInteger &cObIn) const throw() {return GreaterThan(cObIn);};
		ExtBool operator > (int iIn) const throw() {return GreaterThan(iIn);};
		ExtBool operator > (SpecialValues eSpecIn) const throw() {return GreaterThan(eSpecIn);};
		//		greater than or equal operators
		ExtBool operator >= (const ExtInteger &cObIn) const throw() {return GreaterThanOrEqual(cObIn);};
		ExtBool operator >= (int iIn) const throw() {return GreaterThanOrEqual(iIn);};
		ExtBool operator >= (SpecialValues eSpecIn) const throw() {return GreaterThanOrEqual(eSpecIn);};
		//		less than operators
		ExtBool operator < (const ExtInteger &cObIn) const throw() {return LessThan(cObIn);};
		ExtBool operator < (int iIn) const throw() {return LessThan(iIn);};
		ExtBool operator < (SpecialValues eSpecIn) const throw() {return LessThan(eSpecIn);};
		//		less than or equal operator
		ExtBool operator <= (const ExtInteger &cObIn) const throw() {return LessThanOrEqual(cObIn);};
		ExtBool operator <= (int iIn) const throw() {return LessThanOrEqual(iIn);};
		ExtBool operator <= (SpecialValues eSpecIn) const throw() {return LessThanOrEqual(eSpecIn);};
		// Overloaded arithmetic operators
		//		unary negation
		ExtInteger operator - () const throw() {return Negative();};
		//		increment operators
		ExtInteger& operator ++ () throw() {Initialise(Add(1)); return *this;};
		ExtInteger operator ++ (int) throw() {ExtInteger cOut; cOut.Initialise(*this); Initialise(Add(1)); return cOut;};
		//		decrement operators
		ExtInteger& operator -- () throw() {Initialise(Subtract(1)); return *this;};
		ExtInteger operator -- (int) throw() {ExtInteger cOut; cOut.Initialise(*this); Initialise(Subtract(1)); return cOut;};
		//		functional assignment operators
		//			addition
		ExtInteger& operator += (const ExtInteger &cObIn) throw() {Initialise(Add(cObIn)); return *this;};
		ExtInteger& operator += (int iIn) throw() {Initialise(Add(iIn)); return *this;};
		ExtInteger& operator += (SpecialValues eSpecIn) throw() {Initialise(Add(eSpecIn)); return *this;};
		//			subtraction
		ExtInteger& operator -= (const ExtInteger &cObIn) throw() {Initialise(Subtract(cObIn)); return *this;};
		ExtInteger& operator -= (int iIn) throw() {Initialise(Subtract(iIn)); return *this;};
		ExtInteger& operator -= (SpecialValues eSpecIn) throw() {Initialise(Subtract(eSpecIn)); return *this;};
		//			multiplication
		ExtInteger& operator *= (const ExtInteger &cObIn) throw() {Initialise(Multiply(cObIn)); return *this;};
		ExtInteger& operator *= (int iIn) throw() {Initialise(Multiply(iIn)); return *this;};
		ExtInteger& operator *= (SpecialValues eSpecIn) throw() {Initialise(Multiply(eSpecIn)); return *this;};
		//			division
		ExtInteger& operator /= (const ExtInteger &cObIn) throw() {Initialise(Divide(cObIn)); return *this;};
		ExtInteger& operator /= (int iIn) throw() {Initialise(Divide(iIn)); return *this;};
		ExtInteger& operator /= (SpecialValues eSpecIn) throw() {Initialise(Divide(eSpecIn)); return *this;};
		//			modulo
		ExtInteger& operator %= (const ExtInteger &cObIn) throw() {Initialise(Modulo(cObIn)); return *this;};
		ExtInteger& operator %= (int iIn) throw() {Initialise(Modulo(iIn)); return *this;};
		ExtInteger& operator %= (SpecialValues eSpecIn) throw() {Initialise(Modulo(eSpecIn)); return *this;};
		//		normal functional operators
		//			addition
		ExtInteger operator + (const ExtInteger &cObIn) const throw() {return Add(cObIn);};
		ExtInteger operator + (int iIn) const throw() {return Add(iIn);};
		ExtInteger operator + (SpecialValues eSpecIn) const throw() {return Add(eSpecIn);};
		//			subtraction
		ExtInteger operator - (const ExtInteger &cObIn) const throw() {return Subtract(cObIn);};
		ExtInteger operator - (int iIn) const throw() {return Subtract(iIn);};
		ExtInteger operator - (SpecialValues eSpecIn) const throw() {return Subtract(eSpecIn);};
		//			multiplication
		ExtInteger operator * (const ExtInteger &cObIn) const throw() {return Multiply(cObIn);};
		ExtInteger operator * (int iIn) const throw() {return Multiply(iIn);};
		ExtInteger operator * (SpecialValues eSpecIn) const throw() {return Multiply(eSpecIn);};
		//			division
		ExtInteger operator / (const ExtInteger &cObIn) const throw() {return Divide(cObIn);};
		ExtInteger operator / (int iIn) const throw() {return Divide(iIn);};
		ExtInteger operator / (SpecialValues eSpecIn) const throw() {return Divide(eSpecIn);};
		//			modulo
		ExtInteger operator % (const ExtInteger &cObIn) const throw() {return Modulo(cObIn);};
		ExtInteger operator % (int iIn) const throw() {return Modulo(iIn);};
		ExtInteger operator % (SpecialValues eSpecIn) const throw() {return Modulo(eSpecIn);};
	};

	// Overloaded operators for native integer and extended integer interaction
	//		normal arithmetic functions
	inline ExtInteger operator + (int iIn, const ExtInteger &cObIn) throw() {return cObIn.Add(iIn);}
	inline ExtInteger operator - (int iIn, const ExtInteger &cObIn) throw() {return cObIn.Negative().Add(iIn);}
	inline ExtInteger operator * (int iIn, const ExtInteger &cObIn) throw() {return cObIn.Multiply(iIn);}
	inline ExtInteger operator / (int iIn, const ExtInteger &cObIn) throw() {ExtInteger cTemp; cTemp.Initialise(iIn); return cTemp.Divide(cObIn);}
	inline ExtInteger operator % (int iIn, const ExtInteger &cObIn) throw() {ExtInteger cTemp; cTemp.Initialise(iIn); return cTemp.Modulo(cObIn);}
	inline ExtInteger operator + (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.Add(eSpecIn);}
	inline ExtInteger operator - (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.Negative().Add(eSpecIn);}
	inline ExtInteger operator * (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.Multiply(eSpecIn);}
	inline ExtInteger operator / (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {ExtInteger cTemp; cTemp.Initialise(eSpecIn); return cTemp.Divide(cObIn);}
	inline ExtInteger operator % (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {ExtInteger cTemp; cTemp.Initialise(eSpecIn); return cTemp.Modulo(cObIn);}
	//		assignment arithmetic functions
	inline int& operator += (int &iIn, const ExtInteger &cObIn) throw(Exception) {return (iIn += cObIn.GetAsInteger());}
	inline int& operator -= (int &iIn, const ExtInteger &cObIn) throw(Exception) {return (iIn -= cObIn.GetAsInteger());}
	inline int& operator *= (int &iIn, const ExtInteger &cObIn) throw(Exception) {return (iIn *= cObIn.GetAsInteger());}
	inline int& operator /= (int &iIn, const ExtInteger &cObIn) throw(Exception) {return (iIn /= cObIn.GetAsInteger());}
	inline int& operator %= (int &iIn, const ExtInteger &cObIn) throw(Exception) {ExtInteger cTemp; cTemp.Initialise(iIn); return (iIn = cTemp.Modulo(cObIn).GetAsInteger());}
	//		comparison functions
	inline ExtBool operator == (int iIn, const ExtInteger &cObIn) throw() {return cObIn.Equal(iIn);}
	inline ExtBool operator != (int iIn, const ExtInteger &cObIn) throw() {return cObIn.NotEqual(iIn);}
	inline ExtBool operator > (int iIn, const ExtInteger &cObIn) throw() {return cObIn.LessThan(iIn);}
	inline ExtBool operator >= (int iIn, const ExtInteger &cObIn) throw() {return cObIn.LessThanOrEqual(iIn);}
	inline ExtBool operator < (int iIn, const ExtInteger &cObIn) throw() {return cObIn.GreaterThan(iIn);}
	inline ExtBool operator <= (int iIn, const ExtInteger &cObIn) throw() {return cObIn.GreaterThanOrEqual(iIn);}
	inline ExtBool operator == (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.Equal(eSpecIn);}
	inline ExtBool operator != (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.NotEqual(eSpecIn);}
	inline ExtBool operator > (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.LessThan(eSpecIn);}
	inline ExtBool operator >= (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.LessThanOrEqual(eSpecIn);}
	inline ExtBool operator < (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.GreaterThan(eSpecIn);}
	inline ExtBool operator <= (SpecialValues eSpecIn, const ExtInteger &cObIn) throw() {return cObIn.GreaterThanOrEqual(eSpecIn);}

	// -------------------------------
	//  EXTENDED COMPLEX NUMBER CLASS
	// -------------------------------
	// An class for holding complex number.  Both the real and imaginary parts are of ExtDouble and
	// so can hold +Inf, -Inf, NaN and NA values
	class ExtComplex
	{
	private:
		ExtDouble cReal;	// Real part of the complex number
		ExtDouble cImag;	// Imaginary part of the complex number
	public:
		// Empty initialisation function
		ExtComplex() throw() : cReal(), cImag() {};
		void Initialise() throw() {cReal.Initialise(); cImag.Initialise();};
		// Parameterised initialisation function 1
		ExtComplex(const ExtDouble &cRealIn, const ExtDouble &cImagIn) throw() : cReal(cRealIn), cImag(cImagIn) {};
		void Initialise(const ExtDouble &cRealIn, const ExtDouble &cImagIn) throw() {cReal.Initialise(cRealIn); cImag.Initialise(cImagIn);};
		ExtComplex& operator = (const ExtDouble &cRealIn) throw() {Initialise(cRealIn, ExtDouble(0.0)); return *this;};
		// Parameterised initialisation function 2
		ExtComplex(double dRealIn, double dImagIn) throw() : cReal(dRealIn), cImag(dImagIn) {};
		void Initialise(double dRealIn, double dImagIn) throw() {cReal.Initialise(dRealIn); cImag.Initialise(dImagIn);};
		ExtComplex& operator = (double dRealIn) throw() {cReal.Initialise(dRealIn); cImag.Initialise(0.0); return *this;};
		// Copy initialisation function
		ExtComplex(const ExtComplex &cObIn) throw() : cReal(cObIn.cReal), cImag(cObIn.cImag) {};
		void Initialise(const ExtComplex &cObIn) throw() {cReal.Initialise(cObIn.cReal); cImag.Initialise(cObIn.cImag);};
		ExtComplex& operator = (const ExtComplex &cObIn) throw() {Initialise(cObIn); return *this;};
		// Return the components of the complex number
		ExtDouble& GetReal() const throw() {return const_cast<ExtDouble &>(cReal);};
		ExtDouble& GetImaginary() const throw() {return const_cast<ExtDouble &>(cImag);};
		// Return the negative of the calling object
		ExtComplex Negative() const throw() {return ExtComplex(cReal.Negative(), cImag.Negative());};
		// Return the complex conjugate
		ExtComplex Conjugate() const throw() {return ExtComplex(cReal, cImag.Negative());};
		// Arithmetic functions
		//		addition
		ExtComplex Add(const ExtComplex &cObIn) const throw() {return ExtComplex(cReal.Add(cObIn.cReal), cImag.Add(cObIn.cImag));};
		ExtComplex Add(const ExtDouble &cObIn) const throw() {return ExtComplex(cReal.Add(cObIn), cImag);};
		ExtComplex Add(double dIn) const throw() {return ExtComplex(cReal.Add(dIn), cImag);};
		//		subtraction
		ExtComplex Subtract(const ExtComplex &cObIn) const throw() {return ExtComplex(cReal.Subtract(cObIn.cReal), cImag.Subtract(cObIn.cImag));};
		ExtComplex Subtract(const ExtDouble &cObIn) const throw() {return ExtComplex(cReal.Subtract(cObIn), cImag);};
		ExtComplex Subtract(double dIn) const throw() {return ExtComplex(cReal.Subtract(dIn), cImag);};
		//		multiplication
		ExtComplex Multiply(const ExtComplex &cObIn) const throw();
		ExtComplex Multiply(const ExtDouble &cObIn) const throw() {return Multiply(ExtComplex(cObIn, ExtDouble(0.0)));};
		ExtComplex Multiply(double dIn) const throw() {return Multiply(ExtComplex(ExtDouble(dIn), ExtDouble(0.0)));};
		//		division
		ExtComplex Divide(const ExtComplex &cObIn) const throw();
		ExtComplex Divide(const ExtDouble &cObIn) const throw() {return Divide(ExtComplex(cObIn, ExtDouble(0.0)));};
		ExtComplex Divide(double dIn) const throw() {return Divide(ExtComplex(ExtDouble(dIn), ExtDouble(0.0)));};
		// Comparison functions
		//		equal
		ExtBool Equal(const ExtComplex &cObIn) const throw();
		ExtBool Equal(const ExtDouble &cObIn) const throw() {return Equal(ExtComplex(cObIn, ExtDouble(0.0)));};
		ExtBool Equal(double dIn) const throw() {return Equal(ExtComplex(ExtDouble(dIn), ExtDouble(0.0)));};
		//		not equal
		ExtBool NotEqual(const ExtComplex &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool NotEqual(const ExtDouble &cObIn) const throw() {return Equal(cObIn).Inverse();};
		ExtBool NotEqual(double dIn) const throw() {return Equal(dIn).Inverse();};
		// Overloaded comparison operators
		//		equality
		ExtBool operator == (const ExtComplex &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator == (const ExtDouble &cObIn) const throw() {return Equal(cObIn);};
		ExtBool operator == (double dIn) const throw() {return Equal(dIn);};
		//		inequality
		ExtBool operator != (const ExtComplex &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool operator != (const ExtDouble &cObIn) const throw() {return NotEqual(cObIn);};
		ExtBool operator != (double dIn) const throw() {return NotEqual(dIn);};
		// Overloaded arithmetic operators
		//		unary negation
		ExtComplex operator - () const throw() {return Negative();};
		//		functional assignment operators
		//			addition
		ExtComplex& operator += (const ExtComplex &cObIn) throw() {Initialise(Add(cObIn)); return *this;};
		ExtComplex& operator += (const ExtDouble &cObIn) throw() {Initialise(Add(cObIn)); return *this;};
		ExtComplex& operator += (double dIn) throw() {Initialise(Add(dIn)); return *this;};
		//			subtraction
		ExtComplex& operator -= (const ExtComplex &cObIn) throw() {Initialise(Subtract(cObIn)); return *this;};
		ExtComplex& operator -= (const ExtDouble &cObIn) throw() {Initialise(Subtract(cObIn)); return *this;};
		ExtComplex& operator -= (double dIn) throw() {Initialise(Subtract(dIn)); return *this;};
		//			multiplication
		ExtComplex& operator *= (const ExtComplex &cObIn) throw() {Initialise(Multiply(cObIn)); return *this;};
		ExtComplex& operator *= (const ExtDouble &cObIn) throw() {Initialise(Multiply(cObIn)); return *this;};
		ExtComplex& operator *= (double dIn) throw() {Initialise(Multiply(dIn)); return *this;};
		//			division
		ExtComplex& operator /= (const ExtComplex &cObIn) throw() {Initialise(Divide(cObIn)); return *this;};
		ExtComplex& operator /= (const ExtDouble &cObIn) throw() {Initialise(Divide(cObIn)); return *this;};
		ExtComplex& operator /= (double dIn) throw() {Initialise(Divide(dIn)); return *this;};
		//		mormal functional operators
		//			addition
		ExtComplex operator + (const ExtComplex &cObIn) const throw() {return Add(cObIn);};
		ExtComplex operator + (const ExtDouble &cObIn) const throw() {return Add(cObIn);};
		ExtComplex operator + (double dIn) const throw() {return Add(dIn);};
		//			subtraction
		ExtComplex operator - (const ExtComplex &cObIn) const throw() {return Subtract(cObIn);};
		ExtComplex operator - (const ExtDouble &cObIn) const throw() {return Subtract(cObIn);};
		ExtComplex operator - (double dIn) const throw() {return Subtract(dIn);};
		//			multiplication
		ExtComplex operator * (const ExtComplex &cObIn) const throw() {return Multiply(cObIn);};
		ExtComplex operator * (const ExtDouble &cObIn) const throw() {return Multiply(cObIn);};
		ExtComplex operator * (double dIn) const throw() {return Multiply(dIn);};
		//			division
		ExtComplex operator / (const ExtComplex &cObIn) const throw() {return Divide(cObIn);};
		ExtComplex operator / (const ExtDouble &cObIn) const throw() {return Divide(cObIn);};
		ExtComplex operator / (double dIn) const throw() {return Divide(dIn);};
	};

	// Overloaded operators for native double and extended complex number interaction
	//		normal arithmetic functions
	inline ExtComplex operator + (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.Add(cDblIn);}
	inline ExtComplex operator + (double dIn, const ExtComplex &cObIn) throw() {return cObIn.Add(dIn);}
	inline ExtComplex operator - (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.Negative().Add(cDblIn);}
	inline ExtComplex operator - (double dIn, const ExtComplex &cObIn) throw() {return cObIn.Negative().Add(dIn);}
	inline ExtComplex operator * (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.Multiply(cDblIn);}
	inline ExtComplex operator * (double dIn, const ExtComplex &cObIn) throw() {return cObIn.Multiply(dIn);}
	inline ExtComplex operator / (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return ExtComplex(cDblIn, ExtDouble(0.0)).Divide(cCpxIn);}
	inline ExtComplex operator / (double dIn, const ExtComplex &cObIn) throw() {return ExtComplex(ExtDouble(dIn), ExtDouble(0.0)).Divide(cObIn);}
	//		comparison functions
	inline ExtBool operator == (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.Equal(cDblIn);}
	inline ExtBool operator == (double dIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.Equal(dIn);}
	inline ExtBool operator != (const ExtDouble &cDblIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.NotEqual(cDblIn);}
	inline ExtBool operator != (double dIn, const ExtComplex &cCpxIn) throw() {return cCpxIn.NotEqual(dIn);}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif

#endif
