#include "stdafx.h"
#include "RInterface.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )
#endif

namespace RInterface
{
	namespace
	{	// Values to denote habitat suitability / unsuitability
		const int HAB_SUITABLE = 1;
		const int HAB_UNSUITABLE = 0;
	}

	// Initialise the number of R random number generators
	int RRandom::iNumGenerators = 0;

	// Function to find an R object type
	RObjectType CalculateObjectType(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RObjectType eOut = e_NULL;
		SEXPTYPE pType = TYPEOF(pRObIn);
		switch(pType)
		{
		case REALSXP:	// Input object is a vector of doubles
			eOut = e_DBLVEC;
			break;
		case INTSXP:	// Input object is a vectof of integers
			eOut = e_INTVEC;
			break;
		case CPLXSXP:	// Input object is a vector of complex numbers
			eOut = e_CPXVEC;
			break;
		case LGLSXP:	// Input object is a vector of booleans
			eOut = e_BOOLVEC;
			break;
		case STRSXP:	// Input object is a vector of strings
			eOut = e_STRVEC;
			break;
		case VECSXP:	// Input object is a list of some type.  Either a LISP-style list
		case LISTSXP:	// or a generic vector
			eOut = e_LIST;
			break;
		case CLOSXP:	// Input object is a function closure
		case FUNSXP:
		case SPECIALSXP:
		case BUILTINSXP:
			eOut = e_FUNC;
			break;
		case ENVSXP:	// Input object is an environment
			eOut = e_ENV;
			break;
		case NILSXP:	// Input object is a NULL object
			eOut = e_NULL;
			break;
		case DOTSXP:
		case SYMSXP:
		default:		// Input object is not supported by this class library
			StdExtensions::RaiseException("R object is an unsupported type", __LINE__, "RInterface::GetObjectType(SEXP)", __FILE__, StdExtensions::e_DomainError);
			break;
		}
		return eOut;
	}

	// Function to protect and count
	void ProtectAndCount(SEXP pRObIn, size_t &ulIter) throw()
	{
		PROTECT(pRObIn);	// Protect the input object
		++ulIter;			// Increment the protected object count
	}

	// Function to unprotect and recount
	void UnprotectAndCount(size_t ulNum, size_t &ulIter) throw()
	{
		size_t ulMin = 0;
		// Find the minimum value of the number of protected objects
		// and the requested number unprotected objects
		ulMin = ulNum < ulIter ? ulNum : ulIter;
		if(ulMin > 0)
		{
			UNPROTECT(int(ulMin));
		}
		ulIter -= ulMin;
	}

	// Function to unprotect all
	void UnprotectAll(size_t &ulIter) throw()
	{
		if(ulIter > 0)
		{
			UNPROTECT(int(ulIter));
			ulIter = 0;
		}
	}

	// Get a parameter argument
	SEXP GetParameterItem(SEXP pParamList, const StdExtensions::ExtString &cObIn) throw(StdExtensions::Exception)
	{
		char *pTemp = NULL;
		SEXP pOut = R_NilValue;
		size_t ulLength = cObIn.Length();
		if(ulLength > 0)
		{	// Copy the string to a C-style string and then
			// call the relevant parameterised function
			pTemp = StdExtensions::AllocateMemory<char>(ulLength);
			try
			{
				cObIn.RetrieveCString(pTemp, ulLength);
				pOut = GetParameterItem(pParamList, pTemp);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown
				delete [] pTemp;
				pTemp = NULL;
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown
				delete [] pTemp;
				pTemp = NULL;
				StdExtensions::RaiseException("unknown error thrown during retrieval of parameter item", __LINE__, "RInterface::GetParameterIter(SEXP, const StdExtensions::ExtString &)", __FILE__, StdExtensions::e_UnknownError);
			}
			delete [] pTemp;
			pTemp = NULL;
		}
		else
		{	// Invalid search string input
			StdExtensions::RaiseException("invalid search string input", __LINE__, "RInterface::GetParameterItem(SEXP, const StdExtensions::ExtString &)", __FILE__, StdExtensions::e_DomainError);
		}
		return pOut;
	}

	// Get a parameter argument
	SEXP GetParameterItem(SEXP pParamList, const char *pStrIn) throw(StdExtensions::Exception)
	{
		SEXP pNames = R_NilValue;
		SEXP pElmt = R_NilValue;
		int iIter = 0;
		if(pStrIn == NULL)
		{	// Invalid input
			StdExtensions::RaiseException("invalid search string input", __LINE__, "RInterface::GetParameterItem(SEXP, const char *)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(pParamList == R_NilValue)
		{	// Null value entered as a parameter list
			StdExtensions::RaiseException("parameter list is NULL value", __LINE__, "RInterface::GetParameterItem(SEXP, const char *)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(Rf_isList(pParamList))
		{	// The parameter vector is an old-style LISP list
			for(pElmt = CDR(pParamList); pElmt != R_NilValue && strcmp(CHAR(PRINTNAME(TAG(pElmt))), pStrIn) != 0; pElmt = CDR(pElmt))
			{
			}
			if(pElmt != R_NilValue)
			{
				pElmt = CAR(pElmt);
			}
		}
		else if(Rf_isNewList(pParamList))
		{	// The parameter vector is a new style list
			pNames = Rf_getAttrib(pParamList, R_NamesSymbol);
			if(pNames != R_NilValue)
			{
				for(iIter = 0; pElmt == R_NilValue && iIter < int(Rf_length(pParamList)); ++iIter)
				{
					if(strcmp(CHAR(STRING_ELT(pNames, iIter)), pStrIn) == 0)
					{
						pElmt = VECTOR_ELT(pParamList, iIter);
					}
				}
			}
			else
			{	// Parameter list does not contain any names
				StdExtensions::RaiseException("parameter list does not conatin a names attribute", __LINE__, "RInterface::GetParameterItem(SEXP, const char *)", __FILE__, StdExtensions::e_DomainError);
			}
		}
		else
		{	// Parameter list of an inocrrect SEXPTYPE
			StdExtensions::RaiseException("parameter list is an incorrect SEXPTYPE", __LINE__, "RInterface::GetParameterItem(SEXP, const char *)", __FILE__, StdExtensions::e_DomainError);
		}
		// Unable to find the relevant parameter
		if(pElmt == R_NilValue)
		{
			StdExtensions::RaiseException("unable to find requested parameter title in the list", __LINE__, "RInterface::GetParameterItem(SEXP, const char *)", __FILE__, StdExtensions::e_RangeError);
		}
		return pElmt;
	}

	// Write a standard exception to the R console
	void WriteRException(const StdExtensions::Exception &cObIn) throw()
	{
		char pMessage[StdExtensions::EXCEPTION_BUFFSIZE + 1];
		char pRoutine[StdExtensions::EXCEPTION_BUFFSIZE + 1];
		char pFile[StdExtensions::EXCEPTION_BUFFSIZE + 1];
		Rf_error("%s thrown at line %i of routine %s of file %s (code %i)\n",
			cObIn.GetErrorMessage(pMessage, StdExtensions::EXCEPTION_BUFFSIZE + 1),
			int(cObIn.GetErrorLine()),
			cObIn.GetErrorRoutine(pRoutine, StdExtensions::EXCEPTION_BUFFSIZE + 1),
			cObIn.GetErrorFile(pFile, StdExtensions::EXCEPTION_BUFFSIZE + 1),
			int(cObIn.GetErrorClass()));

	}

	// -------------------------------------------------
	//  METHODS FOR THE R RANDOM NUMBER GENERATOR CLASS
	// -------------------------------------------------

	// Add an R generator to the total of current generators
	void RRandom::AddGenerator() throw()
	{
		if(iNumGenerators == 0)
		{	// If this is the first R generator then retrieve
			// the random number state
			GetRNGstate();
		}
		++iNumGenerators;
	}

	// Delete an R generator from the total of current generators
	void RRandom::RemoveGenerator() throw()
	{
		--iNumGenerators;
		if(iNumGenerators <= 0)
		{	// If this is the last R generator then set the
			// random number state
			PutRNGstate();
		}
	}

	// Generate a random uniform number between zero and one
	double RRandom::UniformGen() throw()
	{
		return unif_rand();
	}

	// ---------------------------------------
	//  METHODS FOR THE R OUTPUT STREAM CLASS
	// ---------------------------------------

	// Output integer to the output stream
	void RConsoleOut::Write(int iNum) const throw()
	{
		Rprintf("%i", iNum);
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output double to the output stream
	void RConsoleOut::Write(double dNum) const throw()
	{
		Rprintf("%f", dNum);
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output string to the output stream
	void RConsoleOut::Write(const char *pString) const throw()
	{
		Rprintf(pString);
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output boolean to the output stream
	void RConsoleOut::Write(bool bOut) const throw()
	{
		if(bOut)
		{
			Rprintf("TRUE");
		}
		else
		{
			Rprintf("FALSE");
		}
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output extended integer to the output stream
	void RConsoleOut::Write(const StdExtensions::ExtInteger &cObIn) const throw()
	{
		if(cObIn.IsMissingValue())
		{	// Output is NA
			Rprintf("NA");
		}
		else if(cObIn.IsNotANumber())
		{	// Output is NaN
			Rprintf("NaN");
		}
		else if(cObIn.IsPositiveInfinity())
		{	// Output is +Inf
			Rprintf("+Inf");
		}
		else if(cObIn.IsNegativeInfinity())
		{	// Output is -Inf
			Rprintf("-Inf");
		}
		else
		{
			try
			{	// Output is a standard value
				Rprintf("%i", cObIn.GetAsInteger());
			}
			catch(...)
			{
				Rprintf("NA");
			}
		}
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output extended double to the output stream
	void RConsoleOut::Write(const StdExtensions::ExtDouble &cObIn) const throw()
	{
		if(cObIn.IsMissingValue())
		{	// Output is NA
			Rprintf("NA");
		}
		else if(cObIn.IsNotANumber())
		{	// Output is NaN
			Rprintf("NaN");
		}
		else if(cObIn.IsPositiveInfinity())
		{	// Output is +Inf
			Rprintf("+Inf");
		}
		else if(cObIn.IsNegativeInfinity())
		{	// Output is -Inf
			Rprintf("-Inf");
		}
		else
		{
			try
			{	// Output is a standard value
				Rprintf("%f", cObIn.GetAsDouble());
			}
			catch(...)
			{
				Rprintf("NA");
			}
		}
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output extended complex number to the output stream
	void RConsoleOut::Write(const StdExtensions::ExtComplex &cObIn) const throw()
	{
		Write(cObIn.GetReal());			// Write real component
		Rprintf("+");
		Write(cObIn.GetImaginary());	// Write imaginary component
		Rprintf("i");
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Output extended boolean to the output stream
	void RConsoleOut::Write(const StdExtensions::ExtBool &cObIn) const throw()
	{
		if(cObIn.IsMissingValue())
		{	// Boolean is a missing value
			Rprintf("NA");
		}
		else
		{
			try
			{	// Write the boolean to the output stream
				if(cObIn.GetAsBool())
				{	// Boolean is true
					Rprintf("TRUE");
				}
				else
				{	// Boolean is false
					Rprintf("FALSE");
				}
			}
			catch(...)
			{
				Rprintf("NA");
			}
		}
		// Flush buffer if autoflush is set to 'on'
		if(bAutoFlush)
		{
			Flush();
		}
	}

	// Flush output to the R console
	void RConsoleOut::Flush() const throw()
	{
		RProtectedStack cProt;			// Create protected stack
		SEXP pTemp = R_NilValue;		// Temp SEXP object
		// Create language object
		cProt.Protect(pTemp = Rf_lang1(Rf_install("flush.console")));
		Rf_eval(pTemp, R_GlobalEnv);	// Evaluate the language object
		cProt.UnprotectAll();			// Unprotect the temporary object
	}

	// ----------------------------------------------
	//  METHODS FOR THE PROTECTED STACK LINKER CLASS
	// ----------------------------------------------

	// Initialise a protected stack linker
	void RProtectedStack::RProtectedStackLinker::Initialise(SEXP pObIn, RProtectedStack::RProtectedStackLinker *pNextIn) throw()
	{
		Initialise();										// Unprotect any previous information
		if(pObIn != R_NilValue)
		{
			PROTECT_WITH_INDEX(pObject = pObIn, &iIndx);	// Protect the input object
		}
		else
		{
			pObject = R_NilValue;
		}
		pNext = pNextIn;				// Set the next element of the stack
	}

	// Unprotect the stack object
	void RProtectedStack::RProtectedStackLinker::Initialise() throw()
	{
		if(pObject != R_NilValue)
		{
			UNPROTECT_PTR(pObject);		// Unprotect the object
		}
		pObject = R_NilValue;			// Set the object to NULL
		iIndx = 0;						// Null index
		pNext = NULL;					// Set the next element to NULL
	}

	// ---------------------------------------
	//  METHODS FOR THE PROTECTED STACK CLASS
	// ---------------------------------------

	// Protect a new SEXP object
	PROTECT_INDEX RProtectedStack::Protect(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack::RProtectedStackLinker *pTemp;
		PROTECT_INDEX iIndx = 0;
		pTemp = NULL;
		if(pRObIn != R_NilValue)
		{	// Allocate a new linker object and initialise it using the current function
			pTemp = StdExtensions::AllocateMemory<RProtectedStack::RProtectedStackLinker>();
			pTemp->Initialise(pRObIn, pStart);
			iIndx = pTemp->GetIndex();
			pStart = pTemp;
		}
		// Retrieve the protection index
		return iIndx;
	}

	// Unprotect all protected elements in the stack
	void RProtectedStack::UnprotectAll() throw()
	{	// Iterate over the protected stack
		RProtectedStack::RProtectedStackLinker *pTempOne, *pTempTwo;
		pTempOne = pTempTwo = NULL;
		if(pStart != NULL)
		{
			pTempOne = pStart;
			pTempTwo = pStart->pNext;
			while(pTempOne != NULL)
			{	// Delete each element of the protected stack
				delete pTempOne;
				pTempOne = pTempTwo;
				if(pTempTwo != NULL)
				{
					pTempTwo = pTempTwo->pNext;
				}
			}
			pStart = NULL;
		}
	}

	// Unprotect the last ulIter objects in the stack
	void RProtectedStack::Unprotect(size_t ulIter) throw()
	{
		// Iterate over the protected stack
		RProtectedStack::RProtectedStackLinker *pTempOne, *pTempTwo;
		size_t ulNum = 0;
		pTempOne = pTempTwo = NULL;
		if(pStart != NULL)
		{
			pTempOne = pStart;
			pTempTwo = pStart->pNext;
			for(ulNum = 0; pTempOne != NULL && ulNum < ulIter; ++ulNum)
			{	// Delete each element of the protected stack
				delete pTempOne;
				pTempOne = pTempTwo;
				if(pTempTwo != NULL)
				{
					pTempTwo = pTempTwo->pNext;
				}
			}
			pStart = pTempOne;
		}
	}

	// ---------------------------------------------
	//  METHODS FOR THE DIMENSION INFORMATION CLASS
	// ---------------------------------------------

	// Function to convert a location on a flat array to a set of dimensional coordinates
	void RDimensionInfo::ConvertIterToDimLoc(int iIter, int *pCoordsOut) const throw(StdExtensions::Exception)
	{
		size_t ulIter, ulCurOffSet;
		int iTotal;
		ulIter = ulCurOffSet = 0;
		iTotal = 0;
		if(pDimSize == NULL || ulNumDims == 0)
		{	// Unable to do location convertion of an uninitialised vector
			StdExtensions::RaiseException("unable to convert location to dimensional coordinates in an uninitialised vector", __LINE__, "RInterface::RDimensionInfo::ConvertIterToDimLoc(size_t, int *)", __FILE__, StdExtensions::e_RangeError);
		}
		else if(ulNumDims == 1)
		{	// There is only one dimension and so enforce simple periodic boundary condition
			pCoordsOut[0] = StdExtensions::PeriodicConvert(iIter, 0, pDimSize[0] - 1);
		}
		else
		{	// Calculate the current offset by wrapping the iter on the suppport for the
			// size of the array
			ulCurOffSet = size_t(StdExtensions::PeriodicConvert(iIter, 0, int(Length()) - 1));
			iTotal = int(Length());
			for(ulIter = ulNumDims - 1; ulIter > 0; --ulIter)
			{
				iTotal /= pDimSize[ulIter];							// Calculate the relevant offset
				pCoordsOut[ulIter] = int(ulCurOffSet) / iTotal;		// Use the offset to calculate the coordinate in that dimension
				ulCurOffSet -= iTotal * pCoordsOut[ulIter];			// Update the current offset
			}
			// Set the last dimensional coordinate
			pCoordsOut[0] = int(ulCurOffSet);
		}
	}

	// Function to convert a set of dimensional coordinates to a location on a flat array
	int RDimensionInfo::ConvertDimLocToIter(const int *pCoordsIn) const throw(StdExtensions::Exception)
	{
		size_t ulIter;
		int iOut, iCurOffSet;
		ulIter = 0;
		iOut = iCurOffSet = 0;
		if(pDimSize == NULL || ulNumDims == 0)
		{	// Unable to do location convertion of an uninitialised vector
			StdExtensions::RaiseException("unable to convert location to dimensional coordinates in an uninitialised vector", __LINE__, "RInterface::RDimensionInfo::ConvertIterToDimLoc(size_t, int *)", __FILE__, StdExtensions::e_RangeError);
		}
		else if(ulNumDims == 1)
		{	// There is only one dimension and so enforce simple periodic boundary condition
			iOut = StdExtensions::PeriodicConvert(pCoordsIn[0], 0, pDimSize[0] - 1);
		}
		else
		{	// There are multiple dimensions
			iCurOffSet = 1;
			iOut = 0;
			for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
			{
				iOut += StdExtensions::PeriodicConvert(pCoordsIn[ulIter], 0, pDimSize[ulIter] - 1) * iCurOffSet;
				iCurOffSet *= pDimSize[ulIter];
			}
		}
		return iOut;
	}

	// Copy dimension information from another object
	void RDimensionInfo::CopyDimensionStructure(const RDimensionInfo *pObIn) throw(StdExtensions::Exception)
	{
		if(pObIn == NULL)
		{	// Null object input
			SetDimensionStructure();
		}
		else if(pObIn != this)
		{	// Ensure input object is not pointing to calling object
			SetDimensionStructure(pObIn->ulNumDims, pObIn->pDimSize, pObIn->pDimNames);
		}
	}

	// Set dimension information to null values
	void RDimensionInfo::SetDimensionStructure() throw()
	{
		size_t ulIter = 0;
		if(ulNumDims > 0)
		{	// Delete dimension information
			if(pDimNames != NULL)
			{
				for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
				{	// Delete dimension name vectors
					if(pDimNames[ulIter] != NULL)
					{
						delete [] pDimNames[ulIter];
					}
					pDimNames[ulIter] = NULL;
				}
				delete [] pDimNames;	// Delete the dimension name vector
				pDimNames = NULL;
			}
			delete [] pDimSize;		// Delete the dimension size vector
			pDimSize = NULL;
			ulNumDims = 0;			// Set the dimension size to zero
		}
	}

	// Set dimension information
	void RDimensionInfo::SetDimensionStructure(size_t ulNumDimIn, const int *pDimIn, StdExtensions::ExtString **pNamesIn) throw(StdExtensions::Exception)
	{
		size_t ulIter, ulNamesIter;
		ulIter = ulNamesIter = 0;
		SetDimensionStructure();	// Delete the current dimension structure
		// If at least one dimension is specified
		if(ulNumDimIn > 0 && pDimIn != NULL)
		{	// Set dimension extents
			pDimSize = StdExtensions::AllocateMemory<int>(ulNumDimIn);
			try
			{	// Fill the dimension extent array - checking for invalid values
				for(ulIter = 0; ulIter < ulNumDimIn; ++ulIter)
				{
					if(pDimIn[ulIter] > 0)
					{	// Store valid dimension input value
						pDimSize[ulIter] = pDimIn[ulIter];
					}
					else
					{	// Invalid value given for dimension input
						StdExtensions::RaiseException("invalid value given for input dimension extent", __LINE__, "RInterface::RDimensionInfo::SetDimensionStructure(size_t, const int *, const StdExtensions::ExtString **)", __FILE__, StdExtensions::e_DomainError);
					}
				}
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown during calculation of dimension extents
				delete [] pDimSize;
				pDimSize = NULL;
				throw;
			}
			catch(...)
			{	// Non standard exception thrown during calculation of dimension extents
				delete [] pDimSize;
				pDimSize = NULL;
				StdExtensions::RaiseException("unknown error thrown during calculation of dimension extents", __LINE__, "RInterface::RDimensionsInfo::SetDimensionStructure(size_t, const int *, const StdExtensions::ExtString **)", __FILE__, StdExtensions::e_UnknownError);
			}
			// If dimension names are specified...
			if(pNamesIn != NULL)
			{	// ... allocate memory to store dimension names
				pDimNames = StdExtensions::AllocateMemory<StdExtensions::ExtString *>(ulNumDimIn);
				// ... fill dimension names vector
				try
				{	// Iterate over the dimension vector
					for(ulIter = 0; ulIter < ulNumDimIn; ++ulIter)
					{
						if(pNamesIn[ulIter] == NULL)
						{	// Input names vector is blank
							pDimNames[ulIter] = NULL;
						}
						else
						{	// Input names are present in the vector - copy them across
							pDimNames[ulIter] = StdExtensions::AllocateMemory<StdExtensions::ExtString>(pDimSize[ulIter]);
							for(ulNamesIter = 0; ulNamesIter < size_t(pDimSize[ulIter]); ++ulNamesIter)
							{	// Copy the dimension name
								pDimNames[ulIter][ulNamesIter].Initialise(pNamesIn[ulIter][ulNamesIter]);
							}
						}
					}
				}
				catch(StdExtensions::Exception)
				{	// Standard exception thrown during calculation of dimension extents
					delete [] pDimSize;
					pDimSize = NULL;
					if(ulIter > 0)
					{	// Delete the dimension name arrays that have already been allocated
						for(--ulIter; ulIter > 0; --ulIter)
						{
							delete [] pDimNames[ulIter];
							pDimNames[ulIter] = NULL;
						}
					}
					// Delete the dimension names array
					if(pDimNames != NULL)
					{
						delete [] pDimNames;
						pDimNames = NULL;
					}
					throw;
				}
				catch(...)
				{	// Non-standard exception thrown during calculation of dimension extents
					delete [] pDimSize;
					pDimSize = NULL;
					if(ulIter > 0)
					{	// Delete the dimension name arrays that have already been allocated
						for(--ulIter; ulIter > 0; --ulIter)
						{
							delete [] pDimNames[ulIter];
							pDimNames[ulIter] = NULL;
						}
					}
					// Delete the dimension names array
					if(pDimNames != NULL)
					{
						delete [] pDimNames;
						pDimNames = NULL;
					}
					throw;
				}
			}
			// Set the number of dimensions
			ulNumDims = ulNumDimIn;
		}
	}

	// Retrieve the dimension extent of a given dimension
	size_t RDimensionInfo::GetDimensionExtent(size_t ulDimIter) const throw(StdExtensions::Exception)
	{
		size_t ulOut = 0;
		if(ulNumDims > 0)
		{	// Dimensional information is present - return the relevant extent
			if(ulDimIter >= ulNumDims)
			{
				StdExtensions::RaiseException("reques made for information on a dimension outside of the current range of dimensions", __LINE__, "RInterface::RDimensionInfo::GetDimensionExtent(size_t)", __FILE__, StdExtensions::e_RangeError);
			}
			ulOut = size_t(pDimSize[ulDimIter]);
		}
		else
		{	// If no dimension information is present then throw exception
			StdExtensions::RaiseException("no dimension information present", __LINE__, "RInterface::RDimensionInfo::GetDimensionExtent(size_t)", __FILE__, StdExtensions::e_RangeError);
		}
		return ulOut;
	}

	// Import dimension information from a SEXP object
	void RDimensionInfo::ImportDimensionStructure(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		SEXP pDims, pInDimNames, pCurNames;
		size_t ulIter, ulNameLen, ulNamesIter, ulTotalSize;
		pDims = pInDimNames = pCurNames = R_NilValue;
		ulIter = ulNameLen = ulNamesIter = ulTotalSize = 0;
		SetDimensionStructure();									// Null initialisation of dimension information
		if(pRObIn != R_NilValue)
		{	// Input object is not a NULL value
			pDims = Rf_getAttrib(pRObIn, R_DimSymbol);				// Retrieve the dimension extents
			pInDimNames = Rf_getAttrib(pRObIn, R_DimNamesSymbol);	// Retrieve the dimension names
			// Check to ensure the dimension vector in an integer vector
			if(CalculateObjectType(pDims) != e_INTVEC && CalculateObjectType(pDims) != e_NULL)
			{
				StdExtensions::RaiseException("dimension extent vector is not an integer vector", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
			// Check to ensure the dimension names vector is a new-style list
			if(TYPEOF(pInDimNames) != VECSXP && CalculateObjectType(pInDimNames) != e_NULL)
			{
				StdExtensions::RaiseException("list of dimension names is not a list-type vector", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
			if(pDims == R_NilValue)
			{	// If no dimensional information is present then assume one dimension of
				// length equal to the length of the vector
				pDimSize = StdExtensions::AllocateMemory<int>(1);
				pDimSize[0] = int(Length());						// Size of the one dimension
				if(pInDimNames == R_NilValue)
				{
					pDimNames = NULL;								// Dimension has no names elements
				}
				else
				{	// Set the dimension names
					pCurNames = VECTOR_ELT(pInDimNames, 0);			// Retrieve the first set of dimension
					// Allocate memory for dimension names
					pDimNames = StdExtensions::AllocateMemory<StdExtensions::ExtString *>(1);
					pDimNames[0] = NULL;
					try
					{
						pDimNames[0] = StdExtensions::AllocateMemory<StdExtensions::ExtString>(pDimSize[0]);
						// Copy dimension names across
						for(ulIter = 0; ulIter < size_t(pDimSize[0]); ++ulIter)
						{
							pDimNames[0][ulIter].Initialise(
								CHAR(STRING_ELT(pCurNames, StdExtensions::PeriodicConvert(int(ulIter), 0, int(Rf_length(pCurNames))))));
						}
					}
					catch(StdExtensions::Exception)
					{	// Standard exception thrown - delete dimension information and rethrow
						if(pDimNames != NULL)
						{	// Delete the dimnames
							if(pDimNames[0] != NULL)
							{
								delete [] pDimNames[0];
								pDimNames[0] = NULL;
							}
							delete [] pDimNames;
							pDimNames = NULL;
						}
						// Delete the dimension extents
						delete [] pDimSize;
						pDimSize = NULL;
						throw;
					}
					catch(...)
					{	// Non-standard exception thrown - delete dimension information and rethrow as standard exception
						if(pDimNames != NULL)
						{	// Delete the dimnames
							if(pDimNames[0] != NULL)
							{
								delete [] pDimNames[0];
								pDimNames[0] = NULL;
							}
							delete [] pDimNames;
							pDimNames = NULL;
						}
						// Delete the dimension extents
						delete [] pDimSize;
						pDimSize = NULL;
						StdExtensions::RaiseException("unknown error thrown during allocation of dimension names", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_UnknownError);
					}
				}
				ulNumDims = 1;				// Set dimension number to one
			}
			else
			{	// Dimensional information is present
				// Calculate the number of dimensions
				ulNumDims = size_t(Rf_length(pDims));
				try
				{
					// Allocate memory for dimensional extents
					pDimSize = StdExtensions::AllocateMemory<int>(ulNumDims);
					ulTotalSize = 1;
					// Fill the dimensional extent array with the values from the input dimensions object
					for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
					{
						pDimSize[ulIter] = INTEGER(pDims)[ulIter];
						ulTotalSize *= size_t(pDimSize[ulIter]);
					}
					// Ensure that the total dimensional extent corresponds with the length of the vector
					if(Length() != ulTotalSize)
					{
						delete [] pDimSize;
						pDimSize = NULL;
						StdExtensions::RaiseException("dimensional extents do not corresond with vector length", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_DomainError);
					}
					// Test for the presence of dimension names
					if(pInDimNames == R_NilValue)
					{
						pDimNames = NULL;							// Dimension has no names elements
					}
					else
					{	// Set the dimension names
						// Allocate memory for dimension names
						pDimNames = StdExtensions::AllocateMemory<StdExtensions::ExtString *>(ulNumDims);
						try
						{	// Copy the dimension names across
							for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
							{	// Allocate memory for a list of strings equal to the length of the relevant dimension
								pDimNames[ulIter] = StdExtensions::AllocateMemory<StdExtensions::ExtString>(pDimSize[ulIter]);
								pCurNames = VECTOR_ELT(pInDimNames, int(ulIter));	// Retrieve the current names
								ulNameLen = Rf_length(pCurNames);					// Get the length of the vector of current names
								// Iterate over the vector of dimension names - coping each into the relevant string
								for(ulNamesIter = 0; ulNamesIter < size_t(pDimSize[ulIter]); ++ulNamesIter)
								{
									pDimNames[ulIter][ulNamesIter].Initialise(
										CHAR(STRING_ELT(pCurNames, StdExtensions::PeriodicConvert(int(ulNamesIter), 0, int(ulNameLen)))));
								}
							}
						}
						catch(StdExtensions::Exception)
						{	// Standard exception thrown - delete partially allocated dimension name arrays
							if(ulIter > 0)
							{	// Delete any already allocated vectors
								for(--ulIter; ulIter > 0; --ulIter)
								{
									delete [] pDimNames[ulIter];
									pDimNames[ulIter] = NULL;
								}
								delete [] pDimNames[0];
								pDimNames[0] = NULL;
							}
							// Delete the dimension names array
							delete [] pDimNames;
							pDimNames = NULL;
							throw;
						}
						catch(...)
						{	// Non-standard exception thrown - delete partially allocated dimension name arrays
							if(ulIter > 0)
							{	// Delete any already allocated vectors
								for(--ulIter; ulIter > 0; --ulIter)
								{
									delete [] pDimNames[ulIter];
									pDimNames[ulIter] = NULL;
								}
								delete [] pDimNames[0];
								pDimNames[0] = NULL;
							}
							// Delete the dimension names array
							delete [] pDimNames;
							pDimNames = NULL;
							StdExtensions::RaiseException("unknown error thrown during import of dimensional names", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_UnknownError);
						}
					}
				}
				catch(StdExtensions::Exception)
				{	// Standard exception thrown - reset dimension count to zero before rethrowing
					ulNumDims = 0;
					throw;
				}
				catch(...)
				{	// Non-standard exception thrown - reset dimension count to zero before throwing standard exception
					ulNumDims = 0;
					StdExtensions::RaiseException("unknown error thrown during calculation of dimensional extents", __LINE__, "RInterface::RDimensionInfo::ImportDimensionStructure(SEXP)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Export dimension names as a SEXP object
	SEXP RDimensionInfo::ExportDimensionNamesAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtOut;
		SEXP pOut, pCurNames;
		size_t ulIter, ulStrLength, ulStrIter;
		char *pTempStr;
		pOut = pCurNames = R_NilValue;
		ulIter = ulStrLength = ulStrIter = 0;
		pTempStr = NULL;
		if(pDimNames != NULL)
		{	
			try
			{	// If there are dimension names then ...
				//	... allocate and protect a new list of elements
				cProtOut.Protect(pOut = Rf_allocVector(VECSXP, R_len_t(ulNumDims)));
				for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
				{	//	... allocate and protect a vector of strings
					pCurNames = R_NilValue;
					cProtOut.Protect(pCurNames = Rf_allocVector(STRSXP, pDimSize[ulIter]));
					//	... iterate over the string vector and copy the dimension names
					for(ulStrIter = 0; ulStrIter < size_t(pDimSize[ulIter]); ++ulStrIter)
					{
						// Calculate the length of the string
						ulStrLength = pDimNames[ulIter][ulStrIter].Length() + 1;
						// Allocate memory to hold a temporary C-string
						pTempStr = StdExtensions::AllocateMemory<char>(ulStrLength);
						pDimNames[ulIter][ulStrIter].RetrieveCString(pTempStr, ulStrLength);
						// Set the string element
						SET_STRING_ELT(pCurNames, int(ulStrIter), Rf_mkChar(pTempStr));
						// Delete the temporary string
						delete [] pTempStr;
						pTempStr = NULL;
					}
					// Set the relevant vector element
					SET_VECTOR_ELT(pOut, int(ulIter), pCurNames);
					cProtOut.Unprotect(1);
				}
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - delete temporary string before rethrowing the exception
				if(pTempStr != NULL)
				{
					delete [] pTempStr;
					pTempStr = NULL;
				}
				cProtOut.UnprotectAll();
				pOut = R_NilValue;
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - delete temporary string before rethrowing standard exception
				if(pTempStr != NULL)
				{
					delete [] pTempStr;
					pTempStr = NULL;
				}
				cProtOut.UnprotectAll();
				pOut = R_NilValue;
				StdExtensions::RaiseException("unknown error thrown during copying of dimension names", __LINE__, "RInterface::RDimensionInfo::ExportDimensionNamesAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any temporary SEXP objects
		cProtOut.UnprotectAll();
		return pOut;
	}

	// Export dimension extents as a SEXP object
	SEXP RDimensionInfo::ExportDimensionExtentsAsSEXP() const throw()
	{
		RProtectedStack cProtStack;			// Stack to handle protection
		SEXP pOut = R_NilValue;
		size_t ulIter = 0;
		// Allocate and protect a vector of integers
		cProtStack.Protect(pOut = Rf_allocVector(INTSXP, R_len_t(ulNumDims)));
		for(ulIter = 0; ulIter < ulNumDims; ++ulIter)
		{
			INTEGER(pOut)[ulIter] = pDimSize[ulIter];
		}
		// Unprotect the vector of integers
		cProtStack.UnprotectAll();
		return pOut;
	}

	// --------------------------------------
	//  METHODS FOR THE INTEGER VECTOR CLASS
	// --------------------------------------

	// Null initialisation function
	void RIntegerVector::Initialise() throw()
	{	// Delete the vector
		if(pVector != NULL)
		{
			delete [] pVector;
			pVector = NULL;
		}
		// Set the vector size to zero
		ulVecSize = 0;
		SetDimensionStructure();	// Set the dimension structure to null
	}

	// Copy initialisation function
	void RIntegerVector::Initialise(const RIntegerVector &cObIn) throw(StdExtensions::Exception)
	{
		size_t ulIter = 0;
		if(&cObIn != this)
		{	// Check to ensure that the input object is not the calling object
			Initialise();
			if(cObIn.ulVecSize > 0)	// If input object is a non-null vector
			{
				// Allocate memory for the new vector
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtInteger>(cObIn.ulVecSize);
				try
				{	// Copy the vector across from the input
					ulVecSize = cObIn.ulVecSize;
					// Copy the vector elements across
					for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
					{
						pVector[ulIter].Initialise(cObIn.pVector[ulIter]);
					}
					// Set the dimension structure
					CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
				}
				catch(StdExtensions::Exception)	// Standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					throw;
				}
				catch(std::bad_cast)			// Casting exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("casting error thrown during vector initialisation", __LINE__, "RInterface::RIntegerVector::Initialise(const RInterface::RIntegerVector &)", __FILE__, StdExtensions::e_CastError);
				}
				catch(...)						// Non-standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("unknown error thrown during vector initialisation", __LINE__, "RInterface::RIntegerVector::Initialise(const RInterface::RIntegerVector &)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Initialise the vector of a known length
	void RIntegerVector::Initialise(size_t ulLengthIn) throw(StdExtensions::Exception)
	{
		int pDimsIn[1];
		Initialise();
		pVector = StdExtensions::AllocateMemory<StdExtensions::ExtInteger>(ulLengthIn);
		ulVecSize = ulLengthIn;
		pDimsIn[0] = int(ulVecSize);
		SetDimensionStructure(1, pDimsIn, NULL);
	}

	// Import information from an R SEXP object
	void RIntegerVector::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;	// Protection stack for SEXP objects
		size_t ulIter = 0;
		SEXP pTemp = R_NilValue;
		Initialise();				// Delete the current contents of the vector
		if(pRObIn != R_NilValue)
		{	// Coerce the vector to a real vector
			cProtStack.Protect(pTemp = Rf_coerceVector(pRObIn, REALSXP));
			try
			{	// Allocate memory for the vector
				ulVecSize = size_t(Rf_length(pRObIn));
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtInteger>(ulVecSize);
				// Copy the contents of the array across
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(REAL(pTemp)[ulIter] == R_NaReal)
					{	// Input is NA
						pVector[ulIter].Initialise(StdExtensions::e_MissingValue);
					}
					else if(REAL(pTemp)[ulIter] == R_NaN)
					{	// Input is NaN
						pVector[ulIter].Initialise(StdExtensions::e_NotANumber);
					}
					else if(REAL(pTemp)[ulIter] == R_PosInf)
					{	// Input is +Inf
						pVector[ulIter].Initialise(StdExtensions::e_PositiveInfinity);
					}
					else if(REAL(pTemp)[ulIter] == R_NegInf)
					{	// Input is -Inf
						pVector[ulIter].Initialise(StdExtensions::e_NegativeInfinity);
					}
					else
					{	// Input is a standard input - not a special value
						pVector[ulIter].Initialise(int(REAL(pTemp)[ulIter]));
					}
				}
				// Import dimension structure from SEXP object
				ImportDimensionStructure(pRObIn);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - unprotect temporary SEXP objects before rethrowing
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - unprotect temporary SEXP objects before throwing standard exception
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown error raised during import of R SEXP object", __LINE__, "RInterface::RIntegerVector::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
			}
			cProtStack.UnprotectAll();
		}
	}

	// Export the vector as an R SEXP object
	SEXP RIntegerVector::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;
		SEXP pDimsOut, pNamesOut, pOut;
		size_t ulCount, ulIter;
		pDimsOut = pNamesOut = pOut = R_NilValue;	// Initialise the output SEXP objects
		ulCount = ulIter = 0;
		if(ulVecSize > 0)
		{	// If the vector has some elements
			try
			{	// Protect the output vectors - including the ones for exporting the dimension structure
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());	// Export dimension names
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());	// Export dimension extents
				cProtStack.Protect(pOut = Rf_allocVector(REALSXP, R_len_t(ulVecSize)));	// Allocate output vector
				// Iterate over the vector copying the values over to the output vector
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(pVector[ulIter].IsMissingValue())
					{	// Output value is a missing value
						REAL(pOut)[ulIter] = R_NaReal;
					}
					else if(pVector[ulIter].IsNotANumber())
					{	// Output value is not a number
						REAL(pOut)[ulIter] = R_NaN;
					}
					else if(pVector[ulIter].IsPositiveInfinity())
					{	// Output value is positive infinity
						REAL(pOut)[ulIter] = R_PosInf;
					}
					else if(pVector[ulIter].IsNegativeInfinity())
					{	// Output value is negative infinity
						REAL(pOut)[ulIter] = R_NegInf;
					}
					else
					{	// Output value is a standard value
						REAL(pOut)[ulIter] = pVector[ulIter].GetAsDouble();
					}
				}
				// Set dimensions and dimension names attributes
				Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
			}
			catch(StdExtensions::Exception)
			{	// Standard error thrown - unprotect SEXP object and then rethrow
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard error thrown - unprotect SEXP objects and then rethrow as standard error
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown exception thrown during export of vector", __LINE__, "RInterface::RIntegerVector::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any protected SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Retrieve an element from the integer vector
	StdExtensions::ExtInteger& RIntegerVector::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		if(ulVecSize == 0)
		{
			StdExtensions::RaiseException("unable to retrieve element from uninitialised vector", __LINE__, "RInterface::RIntegerVector::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<StdExtensions::ExtInteger &>(pVector[size_t(StdExtensions::PeriodicConvert(iIter, 0, int(ulVecSize) - 1))]);
	}

	// Retrieve an element from the integer vector
	StdExtensions::ExtInteger& RIntegerVector::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// Function to generate landscapes by hierarchical curdling
	void RIntegerVector::HierarchicalCurdling(size_t ulDimNumIn, size_t ulHierNum, const int *pExtents, int **pHierBoxes, const int *pHierProp, StdExtensions::Random *pRand) throw(StdExtensions::Exception)
	{
		size_t ulTotalSize, ulDimIter, ulHierIter, ulNumUnhab;
		int *pCoords, *pTempCoords, *pTempExtents;		// Vector to store coordinates for updating of the output vector
		int *pTempTempCoords;
		int iSelect, iTileIter;
		ulTotalSize = ulDimIter = ulHierIter = ulNumUnhab = 0;
		pCoords = pTempCoords = pTempExtents = pTempTempCoords = NULL;
		iSelect = iTileIter = 0;
		Initialise();					// Delete the current contents of the vector
		if(ulDimNumIn > 0)
		{	// There is at least one dimension in the input...
			//		... ensure that there is at least one hierarchy
			if(ulHierNum == 0)
			{
				StdExtensions::RaiseException("invalid input for hierarchy number", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the dimension extent array is valid
			else if(pExtents == NULL)
			{
				StdExtensions::RaiseException("invalid input for dimension extent", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the hierarchical envelope extents are valid
			else if(pHierBoxes == NULL)
			{
				StdExtensions::RaiseException("invalid input for hierarchical envelope extents", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the hierarchical proportions are valid
			else if(pHierProp == NULL)
			{
				StdExtensions::RaiseException("invalid input for hierarchical proportions", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the random number generator is valid
			else if(pRand == NULL)
			{
				StdExtensions::RaiseException("invalid input for random number generator", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			else
			{	// Inputs OK start with processing
				// Calculate the total size of the array
				ulTotalSize = 1;
				for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
				{
					if(pExtents[ulDimIter] > 0)
					{
						ulTotalSize *= size_t(pExtents[ulDimIter]);
					}
					else
					{	// Invalid value given for dimension extent
						StdExtensions::RaiseException("invalid value given for dimension extent", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
					}
				}
				try
				{	// Initialise the array to the correct size and dimension structure
					Initialise(ulTotalSize);
					SetDimensionStructure(ulDimNumIn, pExtents);
					// Set all cell values to 'suitable'
					for(ulHierIter = 0; ulHierIter < ulTotalSize; ++ulHierIter)
					{
						GetElement(int(ulHierIter)).Initialise(HAB_SUITABLE);
					}
					// Allocate memory for dimensional coordinates
					pCoords = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					pTempCoords = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					pTempExtents = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					pTempTempCoords = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					// Commit final hierarchy
					ulTotalSize = 1;
					for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
					{	// Set the coordinates and the temporary coordinates to zero
						pCoords[ulDimIter] = pTempCoords[ulDimIter] = 0;
						// Calculate the total number of cells in the last hierarchy that
						// can fit inside the total spatial extent
						if(pHierBoxes[0][ulDimIter] <= 0 ||							// This hierarchy dimension extent must be non-zero and positive
							pExtents[ulDimIter] <= pHierBoxes[0][ulDimIter] ||
							pExtents[ulDimIter] % pHierBoxes[0][ulDimIter] != 0)
						{	// Invalid input for hierarchical envelope
							StdExtensions::RaiseException("invalid value given for dimension extent of hierarchical envelope", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
						}
						else
						{
							ulTotalSize *= size_t(pExtents[ulDimIter] / pHierBoxes[0][ulDimIter]);
						}
					}
					// Keep making landscape unsuitable until hierarchical threshold is reached
					if(pHierProp[0] >= 0)
					{	// Test the value of the hierarchical proportion
						for(ulNumUnhab = 0; int(ulTotalSize) - int(ulNumUnhab) > pHierProp[0]; ++ulNumUnhab)
						{	// Set the temporary coordinates to zero
							for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
							{
								pTempCoords[ulDimIter] = 0;
							}
							// Select a random cell
							iSelect = pRand->DiscreteUniform(0, int(ulTotalSize - 1 - ulNumUnhab));
							// Find the first suitable cell
							while(GetElement(pTempCoords).GetAsInteger() == HAB_UNSUITABLE)
							{	// Calculate a new coordinate
								pTempCoords[ulDimNumIn - 1] += pHierBoxes[0][ulDimNumIn - 1];
								for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 &&
									pTempCoords[ulDimIter] >= pExtents[ulDimIter];
									--ulDimIter)
								{	// Respecify the temporary coordinates
									pTempCoords[ulDimIter] = pCoords[ulDimIter];
									pTempCoords[ulDimIter - 1] += pHierBoxes[0][ulDimIter - 1];
								}
							}
							// Find the randomly selected cell
							for(iTileIter = 0; iTileIter < iSelect;
								// Iterate the tile iterator if the habitat is suitable
								iTileIter += GetElement(pTempCoords).GetAsInteger() == HAB_SUITABLE ? 1 : 0)
							{
								// Calculate a new coordinate
								pTempCoords[ulDimNumIn - 1] += pHierBoxes[0][ulDimNumIn - 1];
								for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 &&
									pTempCoords[ulDimIter] >= pExtents[ulDimIter];
									--ulDimIter)
								{	// Respecify the temporary coordinates
									pTempCoords[ulDimIter] = pCoords[ulDimIter];
									pTempCoords[ulDimIter - 1] += pHierBoxes[0][ulDimIter - 1];
								}
							}
							// Calculate the endpoint of the algorithm
							for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
							{
								pTempTempCoords[ulDimIter] = pTempCoords[ulDimIter];
								pTempExtents[ulDimIter] = pTempCoords[ulDimIter] + pHierBoxes[0][ulDimIter];
							}
							// Allocate all cells in the chosen tile to 'unsuitable'
							while(pTempCoords[0] < pTempExtents[0])
							{
								GetElement(pTempCoords).Initialise(HAB_UNSUITABLE);
								pTempCoords[ulDimNumIn - 1]++;
								// Iterate the temporary coordinates
								for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 && pTempCoords[ulDimIter] >= pTempExtents[ulDimIter]; --ulDimIter)
								{
									pTempCoords[ulDimIter] = pTempTempCoords[ulDimIter];
									pTempCoords[ulDimIter - 1]++;
								}
							}
						}
					}
					else
					{	// Invalid input for hierarchy proportion
						StdExtensions::RaiseException("invalid value given for proportion of suitable habitat at a given hierarchy", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
					}
					// Filter through each of the hierarchies working upwards
					for(ulHierIter = 1; ulHierIter < ulHierNum; ++ulHierIter)
					{	// Ensure the there is an existing hierarchical extent for the current and the next hierarchy
						if(pHierBoxes[ulHierIter] != NULL && pHierBoxes[ulHierIter - 1] != NULL)
						{	// Ensure that the proportion of landscape assigned at the current hierarchy is valid
							if(pHierProp[ulHierIter] >= 0)
							{
								ulTotalSize = 1;
								for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
								{	// Set the coordinates and the temporary coordinates to zero
									pCoords[ulDimIter] = pTempCoords[ulDimIter] = 0;
									// Calculate the total number of cells at this hierarchy that
									// can fit inside the cells of the next hierarchy down
									if(pHierBoxes[ulHierIter][ulDimIter] <= 0 ||										// This hierarchy dimension extent must be non-zero and positive
										pHierBoxes[ulHierIter - 1][ulDimIter] <= 0 ||									// The next hierarchy dimension extent must be non-zero and positive
										pHierBoxes[ulHierIter - 1][ulDimIter] <= pHierBoxes[ulHierIter][ulDimIter] ||	// The next hierarchy dimension extent must be greater than the current one
										pHierBoxes[ulHierIter - 1][ulDimIter] % pHierBoxes[ulHierIter][ulDimIter] != 0)	// The current dimension extent must be a factor of the next one
									{	// Invalid input for hierarchical envelope
										StdExtensions::RaiseException("invalid value given for dimension extent of hierarchical envelope", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
									}
									else
									{
										ulTotalSize *= size_t(pHierBoxes[ulHierIter - 1][ulDimIter] / pHierBoxes[ulHierIter][ulDimIter]);
									}
								}
								// Iterate over the cells of the next hierarchy down
								while(pCoords[0] < pExtents[0])
								{	// If the habitat is suitable then turn some unsuitable
									if(GetElement(pCoords).GetAsInteger() == HAB_SUITABLE)
									{
										// Keep making landscape unsuitable until hierarchical threshold is reached
										for(ulNumUnhab = 0; ulTotalSize - ulNumUnhab > size_t(pHierProp[ulHierIter]); ++ulNumUnhab)
										{	// Set the temporary coodinates to the current coordinates
											for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
											{
												pTempCoords[ulDimIter] = pCoords[ulDimIter];
											}
											// Select a random cell
											iSelect = pRand->DiscreteUniform(0, int(ulTotalSize - 1 - ulNumUnhab));
											// Find the first suitable cell
											while(GetElement(pTempCoords).GetAsInteger() == HAB_UNSUITABLE)
											{	// Calculate a new coordinate
												pTempCoords[ulDimNumIn - 1] += pHierBoxes[ulHierIter][ulDimNumIn - 1];
												for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 &&
													pTempCoords[ulDimIter] >= pCoords[ulDimIter] + pHierBoxes[ulHierIter - 1][ulDimIter];
													--ulDimIter)
												{	// Respecify the temporary coordinates
													pTempCoords[ulDimIter] = pCoords[ulDimIter];
													pTempCoords[ulDimIter - 1] += pHierBoxes[ulHierIter][ulDimIter - 1];
												}
											}
											// Find the randomly selected cell
											for(iTileIter = 0; iTileIter < iSelect;
												// Iterate the tile iterator if the habitat is suitable
												iTileIter += GetElement(pTempCoords).GetAsInteger() == HAB_SUITABLE ? 1 : 0)
											{
												// Calculate a new coordinate
												pTempCoords[ulDimNumIn - 1] += pHierBoxes[ulHierIter][ulDimNumIn - 1];
												for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 &&
													pTempCoords[ulDimIter] >= pCoords[ulDimIter] + pHierBoxes[ulHierIter - 1][ulDimIter];
													--ulDimIter)
												{	// Respecify the temporary coordinates
													pTempCoords[ulDimIter] = pCoords[ulDimIter];
													pTempCoords[ulDimIter - 1] += pHierBoxes[ulHierIter][ulDimIter - 1];
												}
											}
											// Calculate the endpoint of the algorithm
											for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
											{
												pTempTempCoords[ulDimIter] = pTempCoords[ulDimIter];
												pTempExtents[ulDimIter] = pTempCoords[ulDimIter] + pHierBoxes[ulHierIter][ulDimIter];
											}
											// Allocate all cells in the chosen tile to 'unsuitable'
											while(pTempCoords[0] < pTempExtents[0])
											{
												GetElement(pTempCoords).Initialise(HAB_UNSUITABLE);
												// Iterate the temporary coordinates
												pTempCoords[ulDimNumIn - 1]++;
												for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 && pTempCoords[ulDimIter] >= pTempExtents[ulDimIter]; --ulDimIter)
												{
													pTempCoords[ulDimIter] = pTempTempCoords[ulDimIter];
													pTempCoords[ulDimIter - 1]++;
												}
											}
										}
									}
									// Change starting coordinates to the corner of the tile at the next
									// hierarchy down
									pCoords[ulDimNumIn - 1] += pHierBoxes[ulHierIter - 1][ulDimNumIn - 1];
									for(ulDimIter = ulDimNumIn - 1; ulDimIter > 0 && pCoords[ulDimIter] >= pExtents[ulDimIter]; --ulDimIter)
									{
										pCoords[ulDimIter] = 0;
										pCoords[ulDimIter - 1] += pHierBoxes[ulHierIter - 1][ulDimIter - 1];
									}
								}
							}
							else
							{	// Invalid input for hierarchy proportion
								StdExtensions::RaiseException("invalid value given for proportion of suitable habitat at a given hierarchy", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
							}
						}
						else
						{	// Invalid input for hierarchical envelope
							StdExtensions::RaiseException("invalid value given for dimension extent of hierarchical envelope", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
						}
					}
				}
				catch(StdExtensions::Exception)
				{	// Standard error thrown
					Initialise();
					if(pCoords != NULL)
					{
						delete [] pCoords;
						pCoords = NULL;
					}
					if(pTempCoords != NULL)
					{
						delete [] pTempCoords;
						pTempCoords = NULL;
					}
					if(pTempExtents != NULL)
					{
						delete [] pTempExtents;
						pTempExtents = NULL;
					}
					if(pTempTempCoords != NULL)
					{
						delete [] pTempTempCoords;
						pTempTempCoords = NULL;
					}
					throw;
				}
				catch(...)
				{	// Non standard error thrown
					Initialise();
					if(pCoords != NULL)
					{
						delete [] pCoords;
						pCoords = NULL;
					}
					if(pTempCoords != NULL)
					{
						delete [] pTempCoords;
						pTempCoords = NULL;
					}
					if(pTempExtents != NULL)
					{
						delete [] pTempExtents;
						pTempExtents = NULL;
					}
					if(pTempTempCoords != NULL)
					{
						delete [] pTempTempCoords;
						pTempTempCoords = NULL;
					}
					StdExtensions::RaiseException("unknown exception thrown during hierarchical curdling", __LINE__, "RInterface::RIntegerVector::HierarchicalCurdling(size_t, size_t, const int *, int **, const double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_UnknownError);
				}
				// Final garbage collection
				delete [] pCoords;
				delete [] pTempCoords;
				delete [] pTempExtents;
				delete [] pTempTempCoords;
				pCoords = pTempCoords = pTempExtents = pTempTempCoords = NULL;
			}
		}
	}

	// Function to generate simple percolation maps
	void RIntegerVector::PercolationMap(size_t ulDimNumIn, size_t ulNumHabs, const int *pExtents, const int *pProps, StdExtensions::Random *pRand) throw(StdExtensions::Exception)
	{
		size_t ulTotalSize, ulTotalChosen, ulDimIter, ulHabIter, ulChosen, ulSelec, ulTileIter;
		ulTotalSize = ulTotalChosen = ulDimIter = ulHabIter = ulChosen = ulSelec = ulTileIter = 0;
		Initialise();
		if(ulDimNumIn > 0)
		{	// If some dimensions are specifed...
			if(pRand == NULL)
			{	// ... check the random number generator input
				StdExtensions::RaiseException("invalid random number generator entry", __LINE__, "RInterface::RIntegerVector::PercolationMap(size_t, size_t, const int *, const int *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			else if(pExtents == NULL)
			{	// ... check the dimension inputs
				StdExtensions::RaiseException("invalid input for dimensional extents of output", __LINE__, "RInterface::RIntegerVector::PercolationMap(size_t, size_t, const int *, const int *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			else
			{	// Calculate the total size of the outputs
				ulTotalSize = 1;
				for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
				{
					if(pExtents[ulDimIter] <= 0)
					{	// Invalid value given for a dimensional extent
						StdExtensions::RaiseException("invalid value given in dimensional extents of output", __LINE__, "RInterface::RIntegerVector::PercolationMap(size_t, size_t, const int *, const int *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
					}
					else
					{
						ulTotalSize *= size_t(pExtents[ulDimIter]);
					}
				}
				// Resize the output array and set the dimensions accordingly
				Initialise(ulTotalSize);
				SetDimensionStructure(ulDimNumIn, pExtents, NULL);
				// Set all elements to a 'zero' data type
				for(ulDimIter = 0; ulDimIter < ulTotalSize; ++ulDimIter)
				{
					GetElement(int(ulDimIter)).Initialise(0);
				}
				if(pProps != NULL)
				{	// Iterate over the habitat types
					for(ulHabIter = 0; ulHabIter < ulNumHabs; ++ulHabIter)
					{	// Choose pProps[ulHabIter] cells to be of type (ulHabIter + 1)
						for(ulChosen = 0; int(ulChosen) < pProps[ulHabIter] && ulTotalSize > ulChosen; ++ulChosen, ++ulTotalChosen)
						{	// Select a random cell to convert to the current habitat type
							ulSelec = size_t(pRand->DiscreteUniform(0, int(ulTotalSize) - int(ulTotalChosen) - 1));
							// Find the first zero element
							for(ulTileIter = ulDimIter = 0; GetElement(int(ulDimIter)).GetAsInteger() != 0; ++ulDimIter)
							{
							}
							for(; ulTileIter < ulSelec || GetElement(int(ulDimIter)).GetAsInteger() != 0; ++ulDimIter)
							{	// Increment the tile iterator if the cell is zero valued
								ulTileIter += GetElement(int(ulDimIter)).GetAsInteger() == 0 ? 1 : 0;
							}
							// Set the habitat type
							GetElement(int(ulDimIter)).Initialise(int(ulHabIter + 1));
						}
					}
				}
			}
		}
	}

	// -------------------------------------
	//  METHODS FOR THE DOUBLE VECTOR CLASS
	// -------------------------------------

	// Null initialisation function
	void RDoubleVector::Initialise() throw()
	{	// Delete the vector
		if(pVector != NULL)
		{
			delete [] pVector;
			pVector = NULL;
		}
		// Set the vector size to zero
		ulVecSize = 0;
		SetDimensionStructure();	// Set the dimension structure to null
	}

	// Copy initialisation function
	void RDoubleVector::Initialise(const RDoubleVector &cObIn) throw(StdExtensions::Exception)
	{
		size_t ulIter = 0;
		if(&cObIn != this)
		{	// Check to ensure that the input object is not the calling object
			Initialise();
			if(cObIn.ulVecSize > 0) // If input object is a non-null vector
			{	// Allocate memory for the new vector
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtDouble>(cObIn.ulVecSize);
				try
				{	// Copy the vector across from the input
					ulVecSize = cObIn.ulVecSize;
					// Copy the vector elements across
					for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
					{
						pVector[ulIter].Initialise(cObIn.pVector[ulIter]);
					}
					// Set the dimension structure
					CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
				}
				catch(StdExtensions::Exception)	// Standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					throw;
				}
				catch(std::bad_cast)			// Casting exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("casting error thrown during vector initialisation", __LINE__, "RInterface::RDoubleVector::Initialise(const RInterface::RDoubleVector &)", __FILE__, StdExtensions::e_CastError);
				}
				catch(...)						// Non-standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("unknown error thrown during vector initialisation", __LINE__, "RInterface::RDoubleVector::Initialise(const RInterface::RDoubleVector &)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Initialise the vector of a known length
	void RDoubleVector::Initialise(size_t ulLengthIn) throw(StdExtensions::Exception)
	{
		int pDimsIn[1];
		Initialise();
		pVector = StdExtensions::AllocateMemory<StdExtensions::ExtDouble>(ulLengthIn);
		ulVecSize = ulLengthIn;
		pDimsIn[0] = int(ulVecSize);
		SetDimensionStructure(1, pDimsIn, NULL);
	}

	// Import information from an R SEXP object
	void RDoubleVector::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;
		size_t ulIter = 0;
		SEXP pTemp = R_NilValue;
		Initialise();				// Delete the current contents of the vector
		if(pRObIn != R_NilValue)
		{	// Coerce the vector to a real vector
			cProtStack.Protect(pTemp = Rf_coerceVector(pRObIn, REALSXP));
			try
			{	// Allocate memory for the vector
				ulVecSize = size_t(Rf_length(pRObIn));
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtDouble>(ulVecSize);
				// Copy the contents of the array across
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(REAL(pTemp)[ulIter] == R_NaReal)
					{	// Input is NA
						pVector[ulIter].Initialise(StdExtensions::e_MissingValue);
					}
					else if(REAL(pTemp)[ulIter] == R_NaN)
					{	// Input is NaN
						pVector[ulIter].Initialise(StdExtensions::e_NotANumber);
					}
					else if(REAL(pTemp)[ulIter] == R_PosInf)
					{	// Input is +Inf
						pVector[ulIter].Initialise(StdExtensions::e_PositiveInfinity);
					}
					else if(REAL(pTemp)[ulIter] == R_NegInf)
					{	// Input is -Inf
						pVector[ulIter].Initialise(StdExtensions::e_NegativeInfinity);
					}
					else
					{	// Input is a standard input - not a special value
						pVector[ulIter].Initialise(REAL(pTemp)[ulIter]);
					}
				}
				// Import dimension structure from SEXP object
				ImportDimensionStructure(pRObIn);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - unprotect temporary SEXP objects before rethrowing
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - unprotect temporary SEXP objects before throwing standard exception
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown error raised during import of R SEXP object", __LINE__, "RInterface::RDoubleVector::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
			}
			cProtStack.UnprotectAll();
		}
	}

	// Export the vector as an R SEXP object
	SEXP RDoubleVector::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;					// Protection stack for SEXP objects
		SEXP pDimsOut, pNamesOut, pOut;
		size_t ulIter;
		pDimsOut = pNamesOut = pOut = R_NilValue;	// Initialise the output SEXP objects
		ulIter = 0;
		if(ulVecSize > 0)
		{	// If the vector has some elements
			try
			{	// Protect the output vector - including the ones for exporting the dimension structure
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());	// Export dimension names
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());	// Export dimension extents
				cProtStack.Protect(pOut = Rf_allocVector(REALSXP, R_len_t(ulVecSize)));	// Allocate output vector
				// Iterate over the vector copying the values over to the output vector
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(pVector[ulIter].IsMissingValue())
					{	// Output value is a missing value
						REAL(pOut)[ulIter] = R_NaReal;
					}
					else if(pVector[ulIter].IsNotANumber())
					{	// Output value is not a number
						REAL(pOut)[ulIter] = R_NaN;
					}
					else if(pVector[ulIter].IsPositiveInfinity())
					{	// Output value is positivie infinity
						REAL(pOut)[ulIter] = R_PosInf;
					}
					else if(pVector[ulIter].IsNegativeInfinity())
					{	// Output value is negative infinity
						REAL(pOut)[ulIter] = R_NegInf;
					}
					else
					{	// Output value is a standard value
						REAL(pOut)[ulIter] = pVector[ulIter].GetAsDouble();
					}
				}
				// Set dimensions and dimension names attributes
				Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
			}
			catch(StdExtensions::Exception)
			{	// Standard error thrown - unprotect SEXP object and then rethrow
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard error thrown - unprotect SEXP object and then rethrow as standard error
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown exception thrown during export of vector", __LINE__, "RInterface::RDoubleVector::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any protected SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Retrieve an element from the double vector
	StdExtensions::ExtDouble& RDoubleVector::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		if(ulVecSize == 0)
		{
			StdExtensions::RaiseException("unable to retrieve element from uninitialised vector", __LINE__, "RInterface::RDoubleVector::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<StdExtensions::ExtDouble &>(pVector[size_t(StdExtensions::PeriodicConvert(iIter, 0, int(ulVecSize) - 1))]);
	}

	// Retrieve an element from the double vector
	StdExtensions::ExtDouble& RDoubleVector::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// Function that generates a spectral landscape with set Hurst exponents
	//		this function is multidimensional version of the SpectralSynthesisFM2D algorithm
	//		of Peitgen and Saupe (1988) using an extension by Keitt (2000) to allow differing
	//		Hurst exponents in each dimension
	void RDoubleVector::SpectralSynthesis(size_t ulDimNumIn, const int *pExtents, const double *pHursts, StdExtensions::Random *pRand) throw(StdExtensions::Exception)
	{
		size_t ulTotalSize, ulIter, ulTotalBig, ulBigLength, ulDimIter;
		RComplexVector cFFTInOut;		// Complex vector for input/output of FFT
		int *pElement, *pRevElement;	// Arrays to hold dimensional coordinates
		int iHalfLength;
		double dFreq, dTotalFreq, dTotalFreqSq, dCurHurst, dRad, dPhase;
		bool bMids;
		StdExtensions::ExtComplex *pValue;
		ulTotalSize = ulIter = ulTotalBig = ulBigLength = ulDimIter = 0;
		pElement = pRevElement = NULL;
		iHalfLength = 0;
		dFreq = dTotalFreq = dTotalFreqSq = dCurHurst = dRad = dPhase = 0.0;
		bMids = false;
		pValue = NULL;
		Initialise();					// Delete the current contents of the vector
		if(ulDimNumIn > 0)
		{	// There is at least one dimension in the input...
			//		... ensure that the dimension extent array is valid
			if(pExtents == NULL)
			{
				StdExtensions::RaiseException("invalid input for dimensional extents array", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the Hurst exponent array is valid
			else if(pHursts == NULL)
			{
				StdExtensions::RaiseException("invalid input for Hurst exponent array", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the random generator pointer is valid
			else if(pRand == NULL)
			{
				StdExtensions::RaiseException("invalid input for random number generator", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			else
			{
				ulTotalSize = 1;
				ulBigLength = 4;
				//		... calculate the total size of the array
				for(ulIter = 0; ulIter < ulDimNumIn; ++ulIter)
				{
					ulTotalSize *= size_t(pExtents[ulIter]);
					for(; ulBigLength < size_t(pExtents[ulIter]); ulBigLength *= 2)
					{	// Find the smallest number that is a power of two that
					}	// is greater than or equal to the largest dimension extent
				}
				iHalfLength = int(ulBigLength / 2);
				// Initialise an input vector for FFT
				cFFTInOut.Initialise(ulTotalBig = size_t(StdExtensions::PowI(int(ulBigLength), int(ulDimNumIn))));
				try
				{	// Allocate memory space for dimensional coordinates
					pElement = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					pRevElement = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
					{
						pElement[ulDimIter] = int(ulBigLength);
						pRevElement[ulDimIter] = 0;
					}
					// Set dimension structure of the input/output complex vector
					cFFTInOut.SetDimensionStructure(ulDimNumIn, pElement, NULL);
					// Set the size of the calling array
					Initialise(ulTotalSize);
					// Set the dimension structure of the calling array
					SetDimensionStructure(ulDimNumIn, pExtents, NULL);
					dTotalFreq = dTotalFreqSq = dCurHurst = 0.0;
					// Iterate over the vector elements
					for(ulIter = 0; ulIter < ulTotalBig; ++ulIter)
					{	// Check to see whether current element has already been set
						if(cFFTInOut.GetElement(int(ulIter)).GetReal().IsMissingValue() ||
							cFFTInOut.GetElement(int(ulIter)).GetImaginary().IsMissingValue())
						{
							// Retrieve the dimensional coordinates of the vector element
							cFFTInOut.ConvertIterToDimLoc(int(ulIter), pElement);
							dTotalFreq = dTotalFreqSq = dCurHurst = 0.0;
							for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
							{	// Find the reverse coordinates
								pRevElement[ulDimIter] = pElement[ulDimIter] == 0 ?
									0 : int(ulBigLength) - pElement[ulDimIter];
								// Calculate the frequency in dimension ulDimIter
								dFreq = double(pElement[ulDimIter] < pRevElement[ulDimIter] ?
									pElement[ulDimIter] : pRevElement[ulDimIter]);
								// Calculate the current Hurst exponent
								dCurHurst += dFreq * pHursts[ulDimIter];
								// Calculate the total frequency
								dTotalFreq += dFreq;
								// Calculate the total frequency squared
								dTotalFreqSq += dFreq * dFreq;
							}
							if(dTotalFreq > 0.0)
							{
								// Normalise the current Hurst exponent
								dCurHurst /= dTotalFreq;
								// Calculate a random phase
								dPhase = pRand->Uniform(0.0, 2.0 * StdExtensions::EXT_PI);
								// Calculate a random amplitude
								dRad = pow(dTotalFreqSq, -0.25 * (2.0 * dCurHurst + double(ulDimNumIn))) *
									pRand->Normal(0.0, 1.0);
								// Find the current element in the array
								pValue = &(cFFTInOut.GetElement(int(ulIter)));
								// Set the spectral proterties at the current coordinate
								pValue->GetReal().Initialise(dRad * cos(dPhase));
								pValue->GetImaginary().Initialise(dRad * sin(dPhase));
								// Ensure conjugate symmetry
								cFFTInOut.GetElement(pRevElement).Initialise(pValue->Conjugate());
							}
							else
							{	// All frequencies in all dimensions are at zero
								pValue = &(cFFTInOut.GetElement(int(ulIter)));
								pValue->GetReal().Initialise(0.0);
								pValue->GetImaginary().Initialise(0.0);
							}
							// See if all the coordinates are zero or halfway along the dimension legnth...
							bMids = true;
							for(ulDimIter = 0; ulDimIter < ulDimNumIn && bMids; ++ulDimIter)
							{
								bMids = pElement[ulDimIter] == 0 || pElement[ulDimIter] == int(ulBigLength / 2) ? true : false;
							}
							// ... if they are then set the imaginary component to zero
							if(bMids)
							{
								cFFTInOut.GetElement(int(ulIter)).GetImaginary().Initialise(0.0);
							}
						}
					}
					// Peform an inverse fast Fourier transform on the data
					cFFTInOut.Initialise(cFFTInOut.InvFFT());
					// Finally fill the smaller calling array - taking only the real components
					// of the output from the fast Fourier transform
					for(ulIter = 0; ulIter < ulTotalSize; ++ulIter)
					{	// Retrieve the dimensional coordinates
						ConvertIterToDimLoc(int(ulIter), pElement);
						// Copy the real component
						GetElement(int(ulIter)).Initialise(
							cFFTInOut.GetElement(pElement).GetReal());
					}
				}
				catch(StdExtensions::Exception)
				{	// Standard exception thrown - delete the two dimension arrays
					if(pElement != NULL)
					{
						delete [] pElement;
						pElement = NULL;
					}
					if(pRevElement != NULL)
					{
						delete [] pRevElement;
						pRevElement = NULL;
					}
					throw;
				}
				catch(...)
				{	// Non-standard exception thrown - delete the two dimension arrays
					if(pElement != NULL)
					{
						delete [] pElement;
						pElement = NULL;
					}
					if(pRevElement != NULL)
					{
						delete [] pRevElement;
						pRevElement = NULL;
					}
					StdExtensions::RaiseException("unknown exception thrown during spectral synthesis calculation", __LINE__, "StdExtensions::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_UnknownError);
				}
				// Delete the arrays
				delete [] pElement;
				delete [] pRevElement;
				pElement = pRevElement = NULL;
			}
		}
	}

	// --------------------------------------
	//  METHODS FOR THE COMPLEX VECTOR CLASS
	// --------------------------------------

	// Null initialisation function
	void RComplexVector::Initialise() throw()
	{	// Delete the vector
		if(pVector != NULL)
		{
			delete [] pVector;
			pVector = NULL;
		}
		// Set the vector size to zero
		ulVecSize = 0;
		SetDimensionStructure();	// Set the dimension structure to null
	}

	// Copy initialisation function
	void RComplexVector::Initialise(const RComplexVector &cObIn) throw(StdExtensions::Exception)
	{
		size_t ulIter = 0;
		if(&cObIn != this)
		{	// Check to ensure that the input object is not the calling object
			Initialise();
			if(cObIn.ulVecSize > 0)	// If input object is a non-null vector
			{	// Allocate memory for the new vector
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtComplex>(cObIn.ulVecSize);
				try
				{	// Copy the vector across from the input
					ulVecSize = cObIn.ulVecSize;
					// Copy the vector elements across
					for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
					{
						pVector[ulIter].Initialise(cObIn.pVector[ulIter]);
					}
					// Set the dimension structure
					CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
				}
				catch(StdExtensions::Exception)	// Standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					throw;
				}
				catch(std::bad_cast)			// Casting exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("casting error thrown during vector initialisation", __LINE__, "RInterface::RComplexVector::Initialise(const RInterface::RComplexVector &)", __FILE__, StdExtensions::e_CastError);
				}
				catch(...)						// Non-standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("unknown error thrown during vector initialisation", __LINE__, "RInterface::RComplexVector::Initialise(const RInterface::RComplexVector &)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Initialise the vector of a known length
	void RComplexVector::Initialise(size_t ulLengthIn) throw(StdExtensions::Exception)
	{
		int pDimsIn[1];
		Initialise();
		pVector = StdExtensions::AllocateMemory<StdExtensions::ExtComplex>(ulLengthIn);
		ulVecSize = ulLengthIn;
		pDimsIn[0] = int(ulVecSize);
		SetDimensionStructure(1, pDimsIn, NULL);
	}

	// Import information from an R SEXP object
	void RComplexVector::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;	// Protection stack for SEXP objects
		size_t ulIter = 0;
		SEXP pTemp = R_NilValue;
		Rcomplex sInNum;
		Initialise();				// Delete the current contents of the vector
		if(pRObIn != R_NilValue)
		{	// Coerce the vector to a real vector
			cProtStack.Protect(pTemp = Rf_coerceVector(pRObIn, CPLXSXP));
			try
			{	// Allocate memory for the vector
				ulVecSize = size_t(Rf_length(pRObIn));
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtComplex>(ulVecSize);
				// Copy the contents of the array across
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					sInNum = COMPLEX(pTemp)[ulIter];
					// Real component input
					if(sInNum.r == R_NaReal)
					{	// Input is NA
						pVector[ulIter].GetReal().Initialise(StdExtensions::e_MissingValue);
					}
					else if(sInNum.r == R_NaN)
					{	// Input is NaN
						pVector[ulIter].GetReal().Initialise(StdExtensions::e_NotANumber);
					}
					else if(sInNum.r == R_PosInf)
					{	// Input is +Inf
						pVector[ulIter].GetReal().Initialise(StdExtensions::e_PositiveInfinity);
					}
					else if(sInNum.r == R_NegInf)
					{	// Input is -Inf
						pVector[ulIter].GetReal().Initialise(StdExtensions::e_NegativeInfinity);
					}
					else
					{	// Input is a standard input - not a special value
						pVector[ulIter].GetReal().Initialise(sInNum.r);
					}
					// Imaginary component input
					if(sInNum.i == R_NaReal)
					{	// Input is NA
						pVector[ulIter].GetImaginary().Initialise(StdExtensions::e_MissingValue);
					}
					else if(sInNum.i == R_NaN)
					{	// Input is NaN
						pVector[ulIter].GetImaginary().Initialise(StdExtensions::e_NotANumber);
					}
					else if(sInNum.i == R_PosInf)
					{	// Input is +Inf
						pVector[ulIter].GetImaginary().Initialise(StdExtensions::e_PositiveInfinity);
					}
					else if(sInNum.i == R_NegInf)
					{	// Input is -Inf
						pVector[ulIter].GetImaginary().Initialise(StdExtensions::e_NegativeInfinity);
					}
					else
					{	// Input is a standard input - not a special value
						pVector[ulIter].GetImaginary().Initialise(sInNum.i);
					}
				}
				// Import dimension structure from SEXP object
				ImportDimensionStructure(pRObIn);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - unprotect temporary SEXP objects before rethrowing
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - unprotect temporary SEXP objects before throwing standard exception
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown error raised during import of R SEXP object", __LINE__, "RInterface::RComplexVector::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
			}
			cProtStack.UnprotectAll();
		}
	}

	// Export the vector as an R SEXP object
	SEXP RComplexVector::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;
		SEXP pDimsOut, pNamesOut, pOut;
		size_t ulIter;
		Rcomplex sInNum;
		pDimsOut = pNamesOut = pOut = R_NilValue;	// Initialise the output SEXP objects
		ulIter = 0;
		if(ulVecSize > 0)
		{	// If the vector has some elements
			try
			{	// Protect the output vectors - including the ones for exporting the dimension structure
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());	// Export dimension names
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());	// Export dimension extents
				cProtStack.Protect(pOut = Rf_allocVector(CPLXSXP, R_len_t(ulVecSize)));	// Allocate output vector
				// Iterate over the vector copying the values over to the output vector
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{	// Store real part
					if(pVector[ulIter].GetReal().IsMissingValue())
					{	// Output value is a missing value
						sInNum.r = R_NaReal;
					}
					else if(pVector[ulIter].GetReal().IsNotANumber())
					{	// Output value is not a number
						sInNum.r = R_NaN;
					}
					else if(pVector[ulIter].GetReal().IsPositiveInfinity())
					{	// Output value is positive infinity
						sInNum.r = R_PosInf;
					}
					else if(pVector[ulIter].GetReal().IsNegativeInfinity())
					{	// Output value is negative infinity
						sInNum.r = R_NegInf;
					}
					else
					{	// Output value is a standard value
						sInNum.r = pVector[ulIter].GetReal().GetAsDouble();
					}
					// Store imaginary part
					if(pVector[ulIter].GetImaginary().IsMissingValue())
					{	// Output value is a missing value
						sInNum.i = R_NaReal;
					}
					else if(pVector[ulIter].GetImaginary().IsNotANumber())
					{	// Output value is not a number
						sInNum.i = R_NaN;
					}
					else if(pVector[ulIter].GetImaginary().IsPositiveInfinity())
					{	// Output value is positive infinity
						sInNum.i = R_PosInf;
					}
					else if(pVector[ulIter].GetImaginary().IsNegativeInfinity())
					{	// Output value is negative infinity
						sInNum.i = R_NegInf;
					}
					else
					{	// Output value is a standard value
						sInNum.i = pVector[ulIter].GetImaginary().GetAsDouble();
					}
					COMPLEX(pOut)[ulIter] = sInNum;
				}
				// Set dimensions and dimension names attributes
				Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
			}
			catch(StdExtensions::Exception)
			{	// Standard error thrown - unprotect SEXP object and then rethrow
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard error thrown - unprotect SEXP objects and then rethrow as standard error
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown exception thrown during export of vector", __LINE__, "RInterface::RComplexVector::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any protected SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Retrieve an element for the complex vector
	StdExtensions::ExtComplex& RComplexVector::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		if(ulVecSize == 0)
		{
			StdExtensions::RaiseException("unable to retrieve element from uninitialised vector", __LINE__, "RInterface::RComplexVector::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<StdExtensions::ExtComplex &>(pVector[size_t(StdExtensions::PeriodicConvert(iIter, 0, int(ulVecSize) - 1))]);
	}

	// Retrieve an element from the complex vector
	StdExtensions::ExtComplex& RComplexVector::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// Function that calculates the (discrete) fast fourier transform of the target vector
	RComplexVector RComplexVector::FFT(bool bInverse) const throw(StdExtensions::Exception)
	{
		RComplexVector cOut;
		double *pReal, *pImag;		// Temporary arrays to store real and imaginary components in colummn major order
		size_t ulIter;
		ulIter = 0;
		pReal = pImag = NULL;
		if(ulVecSize > 0)
		{	// If there are some elements in the array...
			cOut.Initialise(*this);		// Initialise the output array
			try
			{	// Allocate memory for arrays
				pReal = StdExtensions::AllocateMemory<double>(ulVecSize);		// Allocate memory for holding real component
				pImag = StdExtensions::AllocateMemory<double>(ulVecSize);		// Allocate memory for holding imaginary component
				// Copy the element across 
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{	// Set the relevant location on the output arrays
					pReal[ulIter] = GetElement(int(ulIter)).GetReal().GetAsDouble();
					pImag[ulIter] = GetElement(int(ulIter)).GetImaginary().GetAsDouble();
				}
				// Do the fast Fourier transform
				StdExtensions::FFT(int(ulNumDims), pDimSize, pReal, pImag, !bInverse);
				// Copy the elements from the outputs to the output array
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{	// Set the relevant location on the output arrays
					cOut.GetElement(int(ulIter)).Initialise(pReal[ulIter], pImag[ulIter]);
				}
			}
			catch(StdExtensions::Exception)
			{
				if(pReal != NULL)
				{	// Delete the temporary real array
					delete [] pReal;
				}
				if(pImag != NULL)
				{	// Delete the temporary imaginary array
					delete [] pImag;
				}
				pReal = pImag = NULL;
				throw;
			}
			catch(...)
			{
				if(pReal != NULL)
				{	// Delete the temporary real array
					delete [] pReal;
				}
				if(pImag != NULL)
				{	// Delete the temporary imaginary array
					delete [] pImag;
				}
				pReal = pImag = NULL;
				RaiseException("unknown error thrown during fast Fourier transform", __LINE__, "RInterface::RComplexVector::FFT(bool)", __FILE__, StdExtensions::e_UnknownError);
			}
			if(pReal != NULL)
			{	// Delete the temporary real array
				delete [] pReal;
			}
			if(pImag != NULL)
			{	// Delete the temporary imaginary array
				delete [] pImag;
			}
			pReal = pImag = NULL;
		}
		return cOut;
	}

	// Simulate the spectral density of fractional Brownian motion
	void RComplexVector::FBMSpectralDensitySim(size_t ulDimNumIn, const int *pExtents, const double *pHursts, StdExtensions::Random *pRand) throw(StdExtensions::Exception)
	{
		size_t ulTotalSize, ulIter, ulTotalBig, ulBigLength, ulDimIter;
		int *pElement, *pRevElement;	// Arrays to hold dimensional coordinates
		int iHalfLength;
		double dFreq, dTotalFreq, dTotalFreqSq, dCurHurst, dRad, dPhase;
		bool bMids;
		StdExtensions::ExtComplex *pValue;
		ulTotalSize = ulIter = ulTotalBig = ulBigLength = ulDimIter = 0;
		pElement = pRevElement = NULL;
		iHalfLength = 0;
		dFreq = dTotalFreq = dTotalFreqSq = dCurHurst = dRad = dPhase = 0.0;
		bMids = false;
		pValue = NULL;
		Initialise();					// Delete the current contents of the vector
		if(ulDimNumIn > 0)
		{	// There is at least one dimension in the input...
			//		... ensure that the dimension extent array is valid
			if(pExtents == NULL)
			{
				StdExtensions::RaiseException("invalid input for dimensional extents array", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the Hurst exponent array is valid
			else if(pHursts == NULL)
			{
				StdExtensions::RaiseException("invalid input for Hurst exponent array", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			//		... ensure that the random generator pointer is valid
			else if(pRand == NULL)
			{
				StdExtensions::RaiseException("invalid input for random number generator", __LINE__, "RInterface::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_DomainError);
			}
			else
			{
				ulTotalSize = 1;
				ulBigLength = 4;
				//		... calculate the total size of the array
				for(ulIter = 0; ulIter < ulDimNumIn; ++ulIter)
				{
					ulTotalSize *= size_t(pExtents[ulIter]);
					for(; ulBigLength < size_t(pExtents[ulIter]); ulBigLength *= 2)
					{	// Find the smallest number that is a power of two that
					}	// is greater than or equal to the largest dimension extent
				}
				iHalfLength = int(ulBigLength / 2);
				// Initialise an input vector for FFT
				Initialise(ulTotalBig = size_t(StdExtensions::PowI(int(ulBigLength), int(ulDimNumIn))));
				try
				{	// Allocate memory space for dimensional coordinates
					pElement = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					pRevElement = StdExtensions::AllocateMemory<int>(ulDimNumIn);
					for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
					{
						pElement[ulDimIter] = int(ulBigLength);
						pRevElement[ulDimIter] = 0;
					}
					// Set dimension structure of the input/output complex vector
					SetDimensionStructure(ulDimNumIn, pElement, NULL);
					dTotalFreq = dTotalFreqSq = dCurHurst = 0.0;
					// Iterate over the vector elements
					for(ulIter = 0; ulIter < ulTotalBig; ++ulIter)
					{	// Retrieve the dimensional coordinates of the vector element
						ConvertIterToDimLoc(int(ulIter), pElement);
						dTotalFreq = dTotalFreqSq = dCurHurst = 0.0;
						for(ulDimIter = 0; ulDimIter < ulDimNumIn; ++ulDimIter)
						{	// Find the reverse coordinates
							pRevElement[ulDimIter] = pElement[ulDimIter] == 0 ?
								0 : int(ulBigLength) - pElement[ulDimIter];
							// Calculate the frequency in dimension ulDimIter
							dFreq = double(pElement[ulDimIter] < pRevElement[ulDimIter] ?
								pElement[ulDimIter] : pRevElement[ulDimIter]);
							// Calculate the current Hurst exponent
							dCurHurst += dFreq * pHursts[ulDimIter];
							// Calculate the total frequency
							dTotalFreq += dFreq;
							// Calculate the total frequency squared
							dTotalFreqSq += dFreq * dFreq;
						}
						if(dTotalFreq > 0.0)
						{
							// Normalise the current Hurst exponent
							dCurHurst /= dTotalFreq;
							// Calculate a random phase
							dPhase = pRand->Uniform(0.0, 2.0 * StdExtensions::EXT_PI);
							// Calculate a random amplitude
							dRad = pow(dTotalFreqSq, -(double(ulDimNumIn) + 2.0 * dCurHurst) / 2.0) *
								pRand->Normal(0.0, 1.0);
							// Find the current element in the array
							pValue = &(GetElement(int(ulIter)));
							// Set the spectral proterties at the current coordinate
							pValue->GetReal().Initialise(dRad * cos(dPhase));
							pValue->GetImaginary().Initialise(dRad * sin(dPhase));
							// Ensure conjugate symmetry
							GetElement(pRevElement).Initialise(pValue->Conjugate());
						}
						else
						{	// All frequencies in all dimensions are at zero
							pValue = &(GetElement(int(ulIter)));
							pValue->GetReal().Initialise(0.0);
							pValue->GetImaginary().Initialise(0.0);
						}
						// See if all the coordinates are zero or halfway along the dimension legnth...
						bMids = true;
						for(ulDimIter = 0; ulDimIter < ulDimNumIn && bMids; ++ulDimIter)
						{
							bMids = pElement[ulDimIter] == 0 || pElement[ulDimIter] == int(ulBigLength / 2) ? true : false;
						}
						// ... if they are then set the imaginary component to zero
						if(bMids)
						{
							GetElement(int(ulIter)).GetImaginary().Initialise(0.0);
						}
					}
				}
				catch(StdExtensions::Exception)
				{	// Standard exception thrown - delete the two dimension arrays
					if(pElement != NULL)
					{
						delete [] pElement;
						pElement = NULL;
					}
					if(pRevElement != NULL)
					{
						delete [] pRevElement;
						pRevElement = NULL;
					}
					throw;
				}
				catch(...)
				{	// Non-standard exception thrown - delete the two dimension arrays
					if(pElement != NULL)
					{
						delete [] pElement;
						pElement = NULL;
					}
					if(pRevElement != NULL)
					{
						delete [] pRevElement;
						pRevElement = NULL;
					}
					StdExtensions::RaiseException("unknown exception thrown during spectral synthesis calculation", __LINE__, "StdExtensions::RDoubleVector::SpectralSynthesis(size_t, int *, double *, StdExtensions::Random *)", __FILE__, StdExtensions::e_UnknownError);
				}
				// Delete the arrays
				delete [] pElement;
				delete [] pRevElement;
				pElement = pRevElement = NULL;
			}
		}
	}

	// --------------------------------------
	//  METHODS FOR THE BOOLEAN VECTOR CLASS
	// --------------------------------------

	// Null initialisation function
	void RBoolVector::Initialise() throw()
	{	// Delete the vector
		if(pVector != NULL)
		{
			delete [] pVector;
			pVector = NULL;
		}
		// Set the vector size to zero
		ulVecSize = 0;
		SetDimensionStructure();	// Set the dimension structure to null
	}

	// Copy initialisation function
	void RBoolVector::Initialise(const RBoolVector &cObIn) throw(StdExtensions::Exception)
	{
		size_t ulIter = 0;
		if(&cObIn != this)
		{	// Check to ensure that the input object is not the calling object
			Initialise();
			if(cObIn.ulVecSize > 0)	// If input object is a non-null vector
			{	// Allocate memory for the new vector
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtBool>(cObIn.ulVecSize);
				try
				{	// Copy the vector across from the input
					ulVecSize = cObIn.ulVecSize;
					// Copy the vector elements across
					for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
					{
						pVector[ulIter].Initialise(cObIn.pVector[ulIter]);
					}
					// Set the dimension structure
					CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
				}
				catch(StdExtensions::Exception)	// Standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					throw;
				}
				catch(std::bad_cast)			// Casting exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("casting error thrown during vector initialisation", __LINE__, "RInterface::RBoolVector::Initialise(const RInterface::RBoolVector &)", __FILE__, StdExtensions::e_CastError);
				}
				catch(...)						// Non-standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("unknown error thrown during vector initialisation", __LINE__, "RInterface::RBoolVector::Initialise(const RInterface::RBoolVector &)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Initialise the vector of a known length
	void RBoolVector::Initialise(size_t ulLengthIn) throw(StdExtensions::Exception)
	{
		int pDimsIn[1];
		Initialise();
		pVector = StdExtensions::AllocateMemory<StdExtensions::ExtBool>(ulLengthIn);
		ulVecSize = ulLengthIn;
		pDimsIn[0] = int(ulVecSize);
		SetDimensionStructure(1, pDimsIn, NULL);
	}

	// Import information from an R SEXP object
	void RBoolVector::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;	// Protection stack for SEXP objects
		size_t ulIter = 0;
		SEXP pTemp = R_NilValue;
		Initialise();				// Delete the current contents of the vector
		if(pRObIn != R_NilValue)
		{	// Coerce the vector to a logical vector
			cProtStack.Protect(pTemp = Rf_coerceVector(pRObIn, LGLSXP));
			try
			{	// Allocate memory for the vector
				ulVecSize = size_t(Rf_length(pRObIn));
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtBool>(ulVecSize);
				// Copy the contents of the array across
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(LOGICAL(pTemp)[ulIter] == NA_LOGICAL)
					{	// Input is NA
						pVector[ulIter].Initialise(StdExtensions::e_MissingValue);
					}
					else if(LOGICAL(pTemp)[ulIter] == TRUE)
					{	// Input is true
						pVector[ulIter].Initialise(true);
					}
					else
					{	// Input is false
						pVector[ulIter].Initialise(false);
					}
				}
				// Import dimension structure from SEXP object
				ImportDimensionStructure(pRObIn);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - unprotect temporary SEXP objects before rethrowing
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - unprotect temporary SEXP object before throwing standard exception
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown error raised during import of R SEXP object", __LINE__, "RInterface::RBoolVector::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
			}
			cProtStack.UnprotectAll();
		}
	}

	// Export the vector as an R SEXP object
	SEXP RBoolVector::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;					// Protection stack for SEXP objects
		SEXP pDimsOut, pNamesOut, pOut;
		size_t ulCount, ulIter;
		pDimsOut = pNamesOut = pOut = R_NilValue;	// Initialise the output SEXP objects
		ulCount = ulIter = 0;
		if(ulVecSize > 0)
		{	// If the vector has some elements
			try
			{	// Protect the output vector - including the ones for exporting the dimension structure
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());	// Export dimension names
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());	// Export dimension extents
				cProtStack.Protect(pOut = Rf_allocVector(LGLSXP, R_len_t(ulVecSize)));	// Allocate output vector
				// Iterate over the vector copying the values over to the output vector
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(pVector[ulIter].IsMissingValue())
					{	// Output value is a missing value
						LOGICAL(pOut)[ulIter] = NA_LOGICAL;
					}
					else if(pVector[ulIter].GetAsBool())
					{	// Output is true
						LOGICAL(pOut)[ulIter] = TRUE;
					}
					else
					{	// Output is false
						LOGICAL(pOut)[ulIter] = FALSE;
					}
				}
				// Set dimensions and dimension names attributes
				Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
			}
			catch(StdExtensions::Exception)
			{	// Standard error thrown - unprotect SEXP object and then rethrow
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard error thrown - unprotect SEXP object and then rethrow as standard error
				cProtStack.UnprotectAll();
				StdExtensions::RaiseException("unknown exception thrown during export of vector", __LINE__, "RInterface::RBoolVector::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any protected SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Retrieve an element from the boolean vector
	StdExtensions::ExtBool& RBoolVector::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		if(ulVecSize == 0)
		{
			StdExtensions::RaiseException("unable to retrieve element from uninitialised vector", __LINE__, "RInterface::RBoolVector::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<StdExtensions::ExtBool &>(pVector[size_t(StdExtensions::PeriodicConvert(iIter, 0, int(ulVecSize) - 1))]);
	}

	// Retrieve an element from the boolean vector
	StdExtensions::ExtBool& RBoolVector::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// -------------------------------------
	//  METHODS FOR THE STRING VECTOR CLASS
	// -------------------------------------

	// Null initialisation function
	void RStringVector::Initialise() throw()
	{	// Delete the vector
		if(pVector != NULL)
		{
			delete [] pVector;
			pVector = NULL;
		}
		// Set the vector size to zero
		ulVecSize = 0;
		SetDimensionStructure();	// Set the dimension structure to null
	}

	// Copy initialisation function
	void RStringVector::Initialise(const RStringVector &cObIn) throw(StdExtensions::Exception)
	{
		size_t ulIter = 0;
		if(&cObIn != this)
		{	// Check to ensure that the input object is not the calling object
			Initialise();
			if(cObIn.ulVecSize > 0)	// If input object is a non-null vector
			{	// Allocate memory for the new vector
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtString>(cObIn.ulVecSize);
				try
				{	// Copy the vector across from the input
					ulVecSize = cObIn.ulVecSize;
					// Copy the vector elements across
					for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
					{
						pVector[ulIter].Initialise(cObIn.pVector[ulIter]);
					}
					// Set the dimension structure
					CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
				}
				catch(StdExtensions::Exception)	// Standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					throw;
				}
				catch(std::bad_cast)			// Casting exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("casting error thrown during vector initialisation", __LINE__, "RInterface::RStringVector::Initialise(const RInterface::RStringVector &)", __FILE__, StdExtensions::e_CastError);
				}
				catch(...)						// Non-standard exception thrown
				{	// Delete the partially allocated array
					ulVecSize = 0;
					delete [] pVector;
					pVector = NULL;
					SetDimensionStructure();
					StdExtensions::RaiseException("unknown error thrown during vector initialisation", __LINE__, "RInterface::RStringVector::Initialise(const RInterface::RStringVector &)", __FILE__, StdExtensions::e_UnknownError);
				}
			}
		}
	}

	// Initialise the vector of a known length
	void RStringVector::Initialise(size_t ulLengthIn) throw(StdExtensions::Exception)
	{
		int pDimsIn[1];
		Initialise();
		pVector = StdExtensions::AllocateMemory<StdExtensions::ExtString>(ulLengthIn);
		ulVecSize = ulLengthIn;
		pDimsIn[0] = int(ulVecSize);
		SetDimensionStructure(1, pDimsIn, NULL);
	}

	// Import information from an R SEXP object
	void RStringVector::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;	// Stack to hold protected SEXP objects
		size_t ulIter = 0;
		SEXP pTemp = R_NilValue;
		Initialise();				// Delete the current contents of the vector
		if(pRObIn != R_NilValue)
		{	// Coerce the vector to a string vector
			cProtStack.Protect(pTemp = Rf_coerceVector(pRObIn, STRSXP));
			try
			{	// Allocate memory for the vector
				ulVecSize = size_t(Rf_length(pRObIn));
				pVector = StdExtensions::AllocateMemory<StdExtensions::ExtString>(ulVecSize);
				// Copy the contents of the array across
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(STRING_ELT(pTemp, int(ulIter)) == NA_STRING)
					{	// Input is NA
						pVector[ulIter].Initialise();
					}
					else
					{
						pVector[ulIter].Initialise(CHAR(STRING_ELT(pTemp, int(ulIter))));
					}
				}
				// Import dimension structure from SEXP object
				ImportDimensionStructure(pRObIn);
			}
			catch(StdExtensions::Exception)
			{	// Standard exception thrown - unprotect temporary SEXP object before rethrowing
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard exception thrown - unprotect temporary SEXP object before throwing standard exception
				ulVecSize = 0;
				if(pVector != NULL)
				{
					delete [] pVector;
					pVector = NULL;
				}
				cProtStack.UnprotectAll();;
				StdExtensions::RaiseException("unknown error raised during import of R SEXP object", __LINE__, "RInterface::RStringVector::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
			}
			cProtStack.UnprotectAll();
		}
	}

	// Export the vector as an R SEXP object
	SEXP RStringVector::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;			// Stack to hold protected SEXP objects
		SEXP pDimsOut, pNamesOut, pOut;
		char *pTemp;
		size_t ulIter, ulStrLen;
		pDimsOut = pNamesOut = pOut = R_NilValue;	// Initialise the output SEXP objects
		pTemp = NULL;
		ulIter = ulStrLen = 0;
		if(ulVecSize > 0)
		{	// If the vector has some elements
			try
			{	// Protect the output vector - including the ones for exporting the dimension structure
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());	// Export dimension names
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());	// Export dimension extents
				cProtStack.Protect(pOut = Rf_allocVector(STRSXP, R_len_t(ulVecSize)));	// Allocate output vector
				// Iterate over the vector copying the values over to the output vector
				for(ulIter = 0; ulIter < ulVecSize; ++ulIter)
				{
					if(pVector[ulIter].IsMissingValue())
					{	// Output value is a missing value
						SET_STRING_ELT(pOut, int(ulIter), NA_STRING);
					}
					else
					{	// Output is a normal string type
						ulStrLen = pVector[ulIter].Length() + 1;
						pTemp = StdExtensions::AllocateMemory<char>(ulStrLen);
						pVector[ulIter].RetrieveCString(pTemp, ulStrLen);
						SET_STRING_ELT(pOut, int(ulIter), Rf_mkChar(pTemp));
						delete [] pTemp;
						pTemp = NULL;
					}
				}
				// Set dimensions and dimension names attributes
				Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
			}
			catch(StdExtensions::Exception)
			{	// Standard error thrown - unprotect SEXP object and then rethrow
				if(pTemp != NULL)
				{
					delete [] pTemp;
					pTemp = NULL;
				}
				cProtStack.UnprotectAll();
				throw;
			}
			catch(...)
			{	// Non-standard error thrown - unprotect SEXP object and then rethrow as standard error
				if(pTemp != NULL)
				{
					delete [] pTemp;
					pTemp = NULL;
				}
				cProtStack.UnprotectAll();;
				StdExtensions::RaiseException("unknown exception thrown during export of vector", __LINE__, "RInterface::RStringVector::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect and protected SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Retrieve an element from the string vector
	StdExtensions::ExtString& RStringVector::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		if(ulVecSize == 0)
		{
			StdExtensions::RaiseException("unable to retrieve element from uninitialised vector", __LINE__, "RInterface::RStringVector::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<StdExtensions::ExtString &>(pVector[size_t(StdExtensions::PeriodicConvert(iIter, 0, int(ulVecSize) - 1))]);
	}

	// Retrieve an element from the string vector
	StdExtensions::ExtString& RStringVector::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// -------------------------------------
	//  METHODS FOR THE R ENVIRONMENT CLASS
	// -------------------------------------

	// Initialise an empty environment object
	void REnvironment::Initialise() throw()
	{
		if(pEnvironment != R_NilValue)
		{
			UNPROTECT_PTR(pEnvironment);
		}
		pEnvironment = R_NilValue;
	}

	// Initialise a function object from another function object
	void REnvironment::Initialise(const REnvironment &cObIn) throw()
	{
		Initialise();
		if(pEnvironment != R_NilValue)
		{
			PROTECT(pEnvironment = cObIn.pEnvironment);
		}
	}

	// Initialise an environment object from a SEXP object
	void REnvironment::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		Initialise();
		if(CalculateObjectType(pRObIn) == e_ENV)
		{
			if(pRObIn != R_NilValue)
			{
				PROTECT(pEnvironment = pRObIn);
			}
		}
		else
		{
			StdExtensions::RaiseException("input SEXP object is not an environment", __LINE__, "StdExtensions::REnvironment::Initialise(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
	}

	// Export the environment as an R SEXP object
	SEXP REnvironment::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		return pEnvironment;
	}

	// ------------------------------
	//  METHODS FOR THE LINKER CLASS
	// ------------------------------

	// Allocate a new copy of the input object
	RObject *RList::RListLinker::AllocateCopy(const RObject &cObject) throw(StdExtensions::Exception)
	{
		RObject *pOut;
		pOut = NULL;
		try
		{	// Create a copy of the respective R object
			switch(cObject.GetObjectType())
			{
			case e_NULL:		// Null data type
				pOut = StdExtensions::AllocateMemory<RObject>();
				break;
			case e_LIST:		// List data type
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RList>(dynamic_cast<const RList &>(cObject)));
				break;
			case e_INTVEC:		// Integer vector
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RIntegerVector>(dynamic_cast<const RIntegerVector &>(cObject)));
				break;
			case e_DBLVEC:		// Double vector
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RDoubleVector>(dynamic_cast<const RDoubleVector &>(cObject)));
				break;
			case e_CPXVEC:		// Complex vector
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RComplexVector>(dynamic_cast<const RComplexVector &>(cObject)));
				break;
			case e_STRVEC:		// String vector
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RStringVector>(dynamic_cast<const RStringVector &>(cObject)));
				break;
			case e_FUNC:		// Function
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<RFunction>(dynamic_cast<const RFunction &>(cObject)));
				break;
			case e_ENV:			// Environment
				pOut = dynamic_cast<RObject *>(
					StdExtensions::AllocateMemory<REnvironment>(dynamic_cast<const REnvironment &>(cObject)));
				break;
			default:
				pOut = NULL;
				StdExtensions::RaiseException("unknown R object type", __LINE__, "RInterface::RList::RListLinker::AllocateCopy(const RObject &)", __FILE__, StdExtensions::e_CastError);
				break;
			}
		}
		catch(StdExtensions::Exception)
		{	// Rethrow the standard exception
			throw;
		}
		catch(std::bad_cast)
		{	// Throw a bad cast exception
			StdExtensions::RaiseException("error thrown during pointer recast", __LINE__, "RInterface::RList::RListLinker::AllocateCopy(const RObject &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{	// Unknown exception type thrown
			StdExtensions::RaiseException("unknown error thrown during object allocation", __LINE__, "RInterface::RList::RListLinker::AllocateMemory(const RObject &)", __FILE__, StdExtensions::e_UnknownError);
		}
		return pOut;
	}

	// Parameterised initialisation function
	void RList::RListLinker::Initialise(RList::RListLinker *pBeforeIn, RList::RListLinker *pAfterIn, const RObject &cObject, const char *pTitle) throw(StdExtensions::Exception)
	{	// Delete the old contents of the list linker function
		Initialise();
		cTitle.Initialise(pTitle);					// Initialise the title of the list element
		pObject = AllocateCopy(cObject);			// Create a copy of the input object
		// Ensure that linkers link to each other correctly
		if(pBeforeIn != NULL)
		{
			pBeforeIn->pAfter = this;
		}
		if(pAfterIn != NULL)
		{
			pAfterIn->pBefore = this;
		}
		pBefore = pBeforeIn;
		pAfter = pAfterIn;
	}

	// Null initialisation function
	void RList::RListLinker::Initialise() throw()
	{
		delete pObject;
		pObject = NULL;			// Delete the object that the linker points too
		cTitle.Initialise();	// Delete the title
		if(pBefore != NULL)
		{						// Set the previous object to the next one
			pBefore->pAfter = pAfter;
		}
		if(pAfter != NULL)
		{						// Set the next object to the previous one
			pAfter->pBefore = pBefore;
		}
		pBefore = pAfter = NULL;
	}

	// -------------------------------------
	//  METHODS FOR THE RLISTITERATOR CLASS
	// -------------------------------------

	// Null initialisation function
	void RList::RListIterator::Initialise() throw()
	{
		size_t ulIter = 0;
		if(pParent != NULL)
		{	// The iterator has a parent
			for(ulIter = 0; ulIter < pParent->ulIterCacheSize; ++ulIter)
			{	// NULL out the iterator's entry in the parent cache
				if(pParent->pIterCache[ulIter] == this)
				{
					pParent->pIterCache[ulIter] = NULL;
					break;
				}
			}
		}
		pParent = NULL;
		pCurElement = NULL;
	}

	// Copy initialisation function
	void RList::RListIterator::Initialise(const RList::RListIterator &cObIn) throw(StdExtensions::Exception)
	{
		if(&cObIn != this)
		{	// Null initialisation
			Initialise();
			if(cObIn.pParent != NULL)
			{
				cObIn.pParent->AddIteratorToCache(this);	// Add the current iterator to the parent's cache
				pParent = cObIn.pParent;					// Set the parent list pointer
				pCurElement = cObIn.pCurElement;			// Set the pointer to the current element
			}
		}
	}

	// Retrieve the object the iterator is currently pointing to
	RObject& RList::RListIterator::GetObject() const throw(StdExtensions::Exception)
	{
		if(pCurElement == NULL || pCurElement->GetObject() == NULL)
		{	// Throw an error if there is no object
			StdExtensions::RaiseException("iterator does not point to a valid R object", __LINE__, "RInterface::RList::RListIterator::GetObject()", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<RObject &>(*(pCurElement->GetObject()));
	}

	// Increment the iterator by one position
	void RList::RListIterator::Increment() throw()
	{	// If the iterator is pointing to something the increment it
		if(pCurElement != NULL)
		{
			pCurElement = pCurElement->GetNext();
			if(pCurElement == NULL)
			{	// If the iterator has left the edge of the list then
				// call the null-initialisation function
				Initialise();
			}
		}
	}

	// Increment the iterator by a number of positions
	void RList::RListIterator::Increment(int iIter) throw()
	{
		int iSubIter = 0;
		int iAbsIter = 0;
		if(iIter != 0)
		{
			iAbsIter = abs(iIter);
			if(iIter > 0)
			{	// Increment the iterator
				for(iSubIter = 0; iSubIter < iAbsIter; ++iSubIter)
				{
					Increment();
				}
			}
			else
			{	// Decrement the iterator
				for(iSubIter = 0; iSubIter < iAbsIter; ++iSubIter)
				{
					Decrement();
				}
			}
		}
	}

	// Decrement the iterator by one position
	void RList::RListIterator::Decrement() throw()
	{	// If the iterator is pointing to something then decrement it
		if(pCurElement != NULL)
		{
			pCurElement = pCurElement->GetPrevious();
			if(pCurElement == NULL)
			{	// If the iterator has left the edge of the list then
				// call the null-initialisation function
				Initialise();
			}
		}
	}

	// Get the title of the current object
	char *RList::RListIterator::GetTitle(char *pStringOut, size_t ulSize) const throw(StdExtensions::Exception)
	{
		char *pOut;
		pOut = NULL;
		if(pCurElement != NULL)
		{
			pOut = pCurElement->GetTitle(pStringOut, ulSize);
		}
		else
		{	// Throw exception if iterator is invalid
			StdExtensions::RaiseException("unable to return title of an invalid iterator", __LINE__, "RInterface::RList::RListIterator::GetTitle(char *, size_t)", __FILE__, StdExtensions::e_RangeError);
		}
		return pOut;
	}

	// -----------------------------
	//  METHODS FOR THE RLIST CLASS
	// -----------------------------

	// Add an iterator to the cache
	void RList::AddIteratorToCache(const RList::RListIterator *pIterIn) const throw(StdExtensions::Exception)
	{
		size_t ulNonNull, ulIter;
		RList::RListIterator **pTempCache;
		ulNonNull = ulIter = 0;
		pTempCache = NULL;
		if(pIterCache == NULL)
		{	// Initialise the cache if it is previously uninitialised
			pIterCache = StdExtensions::AllocateMemory<RListIterator *>(1);
			pIterCache[0] = const_cast<RList::RListIterator *>(pIterIn);
			ulIterCacheSize = 1;
		}
		else
		{	// Iterate over the cache - calculating the number of non-null elements
			for(ulIter = ulNonNull = 0; ulIter < ulIterCacheSize; ++ulIter)
			{
				if(pIterCache[ulIter] != NULL)
				{
					++ulNonNull;
				}
			}
			// Create a temporary cache large enough to store the non-null elements of
			// the current cache plus the new iterator
			pTempCache = StdExtensions::AllocateMemory<RList::RListIterator *>(ulNonNull + 1);
			// Copy the contents of the old cache to the new cache
			for(ulIter = ulNonNull = 0; ulIter < ulIterCacheSize; ++ulIter)
			{
				if(pIterCache[ulIter] != NULL)
				{
					pTempCache[ulNonNull++] = pIterCache[ulIter];
				}
			}
			// Add the new iterator to the cache
			pTempCache[ulNonNull] = const_cast<RList::RListIterator *>(pIterIn);
			// Delete the old cache
			delete [] pIterCache;
			// Copy the old cache to the new cache
			pIterCache = pTempCache;
			ulIterCacheSize = ulNonNull + 1;
		}
	}

	// Null initialisation functions
	void RList::Initialise() throw()
	{	// Pointers to list linker used in the deletion process
		RListLinker *pTemp;
		size_t ulIter = 0;
		pTemp = NULL;
		// Delete the iterator cache
		if(pIterCache != NULL)
		{	// Set any active iterators for the list to null
			for(ulIter = 0; ulIter < ulIterCacheSize; ++ulIter)
			{
				if(pIterCache[ulIter] != NULL)
				{
					pIterCache[ulIter]->Initialise();
				}
			}
			delete [] pIterCache;
		}
		ulIterCacheSize = 0;
		// Delete the list elements
		if(pStart != NULL)
		{	// Keep deleting the second element in the list until there is no second element
			for(pTemp = pStart->pAfter; pTemp != NULL; pTemp = pStart->pAfter)
			{
				delete pTemp;
				pTemp = NULL;
			}
			// Delete the first element of the list
			delete pStart;
		}
		pStart = pFinish = NULL;
		// Set a null dimension structure
		SetDimensionStructure();
	}

	// Copy initialisation function
	void RList::Initialise(const RList &cObIn) throw(StdExtensions::Exception)
	{
		RList::RListLinker *pTemp;
		pTemp = NULL;
		if(&cObIn != this)
		{	// Delete the existing contents
			Initialise();
			// Copy the list elements to the current object
			for(pTemp = cObIn.pStart; pTemp != NULL; pTemp = pTemp->pAfter)
			{
				if(pTemp->GetTitle().IsMissingValue())
				{
					AddElementToBack(*(pTemp->pObject));
				}
				else
				{
					AddElementToBack(*(pTemp->pObject), pTemp->GetTitle());
				}
			}
		}
		try
		{	// Copy the dimension structure
			CopyDimensionStructure(dynamic_cast<const RDimensionInfo *>(&cObIn));
		}
		catch(StdExtensions::Exception)
		{	// Rethrow standard exceptions
			throw;
		}
		catch(std::bad_cast)
		{	// Throw bad cast exception as standard exception
			StdExtensions::RaiseException("error thrown in pointer recasting during copying of dimension structure", __LINE__, "RInterface::RDimensionInfo::Initialise(const RInterface::RList &)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{	// Throw an unknown exception type as standard exception
			StdExtensions::RaiseException("unknown error thrown during copying of dimension structure", __LINE__, "RInterface::RDimensionInfo::Initialise(const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
	}

	// Add an element to the front of the list
	void RList::AddElementToFront(const RObject &cObIn, const StdExtensions::ExtString &cTitle) throw(StdExtensions::Exception)
	{
		RList::RListLinker *pTemp;
		int pNewDim[1];				// Array holding new dimension length
		pTemp = StdExtensions::AllocateMemory<RList::RListLinker>();
		char *pTempStr;
		pNewDim[0] = 0;
		pTempStr = NULL;
		try
		{	// Allocate memory for a temporary string
			pTempStr = StdExtensions::AllocateMemory<char>(cTitle.Length() + 1);
			cTitle.RetrieveCString(pTempStr, cTitle.Length() + 1);
			// Initialise a new linker object
			pTemp->Initialise(NULL, pStart, cObIn, pTempStr);
			pStart = pTemp;			// Set the new start pointer
			if(pFinish == NULL)
			{	// Set the new finish pointer if the list was previously empty
				pFinish = pTemp;
			}
			// Create a new dimension extent to accomodate the new list
			pNewDim[0] = int(Length());
			SetDimensionStructure(1, pNewDim, NULL);
		}
		catch(StdExtensions::Exception)
		{	// Catch standard error - delete temporary object before rethrowing
			delete pTemp;
			if(pTempStr != NULL)
			{
				delete [] pTempStr;
				pTempStr = NULL;
			}
			throw;
		}
		catch(...)
		{	// Catch non standard error - delete temporary object before rethrowing
			delete pTemp;
			if(pTempStr != NULL)
			{
				delete [] pTempStr;
				pTempStr = NULL;
			}
			StdExtensions::RaiseException("unknown exception thrown during addition of element to RList", __LINE__, "RInterface::RList::AddElementToFront(const RObject &)", __FILE__, StdExtensions::e_UnknownError);
		}
		delete [] pTempStr;
		pTempStr = NULL;
	}

	// Add an element to the back of the list
	void RList::AddElementToBack(const RObject &cObIn, const StdExtensions::ExtString &cTitle) throw(StdExtensions::Exception)
	{
		RList::RListLinker *pTemp;
		int pNewDim[1];				// Array holding new dimension length
		pTemp = StdExtensions::AllocateMemory<RList::RListLinker>();
		char *pTempStr;
		pNewDim[0] = 0;
		pTempStr = NULL;
		try
		{	// Allocate memory for a temporary string
			pTempStr = StdExtensions::AllocateMemory<char>(cTitle.Length() + 1);
			cTitle.RetrieveCString(pTempStr, cTitle.Length() + 1);
			// Initialise a new linker object
			pTemp->Initialise(pFinish, NULL, cObIn, pTempStr);
			pFinish = pTemp;			// Set the new start pointer
			if(pStart == NULL)
			{	// Set the new finish pointer if the list was previously empty
				pStart = pTemp;
			}
			// Create a new dimension extent to accomodate the new list
			pNewDim[0] = int(Length());
			SetDimensionStructure(1, pNewDim, NULL);
		}
		catch(StdExtensions::Exception)
		{	// Catch standard error - delete temporary object before rethrowing
			delete pTemp;
			if(pTempStr != NULL)
			{
				delete [] pTempStr;
				pTempStr = NULL;
			}
			throw;
		}
		catch(...)
		{	// Catch non standard error - delete temporary object before rethrowing
			delete pTemp;
			if(pTempStr != NULL)
			{
				delete [] pTempStr;
				pTempStr = NULL;
			}
			StdExtensions::RaiseException("unknown exception thrown during addition of element to RList", __LINE__, "RInterface::RList::AddElementToBack(const RObject &)", __FILE__, StdExtensions::e_UnknownError);
		}
		delete [] pTempStr;
		pTempStr = NULL;
	}

	// Import information from an R SEXP object
	void RList::ImportFromSEXP(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		Initialise();
		RDoubleVector cDblVec, cTempDblVec;		// Temporary vectors of doubles for input
		RIntegerVector cIntVec, cTempIntVec;	// Temporary vectors of integers for input
		RComplexVector cCpxVec, cTempCpxVec;	// Temporary vectors of complex values for input
		RStringVector cStrVec, cTempStrVec;		// Temporary vectors of strings for input
		RBoolVector cLglVec, cTempLglVec;		// Temporary vectors of boolean for input
		RFunction cFunc;						// Temporary function for input
		REnvironment cEnv;						// Temporary environment for input
		RList cList;							// Temporary list for input
		RObject cNull;							// Null object
		// const char *pCurTitle;
		SEXP pNames, pCur;
		size_t ulIter, ulLength;
		ulIter = ulLength = 0;
		pNames = pCur = R_NilValue;
		try
		{
			switch(TYPEOF(pRObIn))
			{
			case VECSXP:	// List is a generic vector
				ulLength = Rf_length(pRObIn);
				pNames = Rf_getAttrib(pRObIn, R_NamesSymbol);
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{	// Iterate over the generic vector - importing each element
					pCur = VECTOR_ELT(pRObIn, int(ulIter));
					// pCurTitle = CHAR(STRING_ELT(pNames, int(ulIter)));
					switch(CalculateObjectType(pCur))
					{
					case e_NULL:		// Input element is a null element
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(cNull, CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(cNull);
						}
						break;
					case e_LIST:		// Input element is a list
						cList.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cList), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cList));
						}
						break;
					case e_INTVEC:		// Input element is an integer vector
						cIntVec.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec));
						}
						break;
					case e_DBLVEC:		// Input element is a double vector
						cDblVec.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec));
						}
						break;
					case e_CPXVEC:		// Input element is a complex vector
						cCpxVec.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec));
						}
						break;
					case e_STRVEC:		// Input element is a string vector
						cStrVec.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec));
						}
						break;
					case e_BOOLVEC:		// Input element is a boolean vector
						cLglVec.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec));
						}
						break;
					case e_FUNC:		// Input element is a function
						cFunc.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc));
						}
						break;
					case e_ENV:			// Input element is an environment
						cEnv.ImportFromSEXP(pCur);
						if(pNames != R_NilValue && STRING_ELT(pNames, int(ulIter)) != R_NilValue && strcmp(CHAR(STRING_ELT(pNames, int(ulIter))), "") != 0)
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv), CHAR(STRING_ELT(pNames, int(ulIter))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv));
						}
						break;
					default:			// Input element is an unsupported type
						StdExtensions::RaiseException("R object is an unsupported type", __LINE__, "RInterface::RList::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_DomainError);
						break;
					}
				}
				break;
			case LISTSXP:	// List is a LISP-style list
				for(pCur = CDR(pRObIn); pCur != R_NilValue; pCur = CDR(pCur))
				{	// Iterate over the pairlist - importing each element
					switch(CalculateObjectType(CAR(pCur)))
					{
					case e_NULL:		// Input element is a null object
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(cNull, CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(cNull);
						}
						break;
					case e_LIST:		// Input element is a list
						cList.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cList), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cList));
						}
						break;
					case e_INTVEC:		// Input element is an integer vector
						cIntVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec));
						}
						break;
					case e_DBLVEC:		// Input element is a double vector
						cDblVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec));
						}
						break;
					case e_CPXVEC:		// Input element is a complex vector
						cCpxVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec));
						}
						break;
					case e_STRVEC:		// Input element is a string vector
						cStrVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec));
						}
						break;
					case e_BOOLVEC:		// Input element is a boolean vector
						cLglVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec));
						}
						break;
					case e_FUNC:		// Input element is a function
						cFunc.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc));
						}
						break;
					case e_ENV:			// Input element is an environment
						cEnv.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv));
						}
						break;
					default:			// Input element is an unsupported type
						StdExtensions::RaiseException("R object is an unsupported type", __LINE__, "RInterface::RList::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_DomainError);
						break;
					}
				}
				break;
			case REALSXP:	// Input is a vector of doubles
				cDblVec.ImportFromSEXP(pRObIn);
				cTempDblVec.Initialise(1);
				ulLength = cDblVec.Length();
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{
					cTempDblVec.GetElement(0).Initialise(cDblVec.GetElement(int(ulIter)));
					AddElementToBack(dynamic_cast<RObject &>(cTempDblVec));
				}
				break;
			case INTSXP:	// Input is a vector of integers
				cIntVec.ImportFromSEXP(pRObIn);
				cTempIntVec.Initialise(1);
				ulLength = cIntVec.Length();
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{
					cTempIntVec.GetElement(0).Initialise(cIntVec.GetElement(int(ulIter)));
					AddElementToBack(dynamic_cast<RObject &>(cTempIntVec));
				}
				break;
			case CPLXSXP:	// Input is a vector of complex numbers
				cCpxVec.ImportFromSEXP(pRObIn);
				cTempCpxVec.Initialise(1);
				ulLength = cCpxVec.Length();
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{
					cTempCpxVec.GetElement(0).Initialise(cCpxVec.GetElement(int(ulIter)));
					AddElementToBack(dynamic_cast<RObject &>(cTempCpxVec));
				}
				break;
			case LGLSXP:	// Input object is a vector of booleans
				cLglVec.ImportFromSEXP(pRObIn);
				cTempLglVec.Initialise(1);
				ulLength = cLglVec.Length();
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{
					cTempLglVec.GetElement(0).Initialise(cLglVec.GetElement(int(ulIter)));
					AddElementToBack(dynamic_cast<RObject &>(cTempLglVec));
				}
				break;
			case STRSXP:	// Input object is a vector of strings
				cStrVec.ImportFromSEXP(pRObIn);
				cTempStrVec.Initialise(1);
				ulLength = cStrVec.Length();
				for(ulIter = 0; ulIter < ulLength; ++ulIter)
				{
					cTempStrVec.GetElement(0).Initialise(cStrVec.GetElement(int(ulIter)));
					AddElementToBack(dynamic_cast<RObject &>(cTempStrVec));
				}
				break;
			case CLOSXP:	// Input is a function closure
				cFunc.ImportFromSEXP(pRObIn);
				AddElementToBack(dynamic_cast<RObject &>(cFunc));
				break;
			case ENVSXP:	// Input is an environment
				cEnv.ImportFromSEXP(pRObIn);
				AddElementToBack(dynamic_cast<RObject &>(cEnv));
				break;
			case NILSXP:	// Input object is a NULL object
				AddElementToBack(dynamic_cast<RObject &>(cNull));
				break;
			case DOTSXP:
			case SYMSXP:
			default:		// Input object is not supported by this class library
				StdExtensions::RaiseException("R object is an unsupported type", __LINE__, "RInterface::RList::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_DomainError);
				break;
			}
			// Import a dimension structure from the input object
			ImportDimensionStructure(pRObIn);
		}
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("error thrown during casting process", __LINE__, "RInterface::RList::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during import", __LINE__, "RInterface::RList::ImportFromSEXP(SEXP)", __FILE__, StdExtensions::e_UnknownError);
		}
	}

	void RList::FillFromExternalParameters(SEXP pRObIn) throw(StdExtensions::Exception)
	{
		Initialise();
		RDoubleVector cDblVec;					// Temporary vectors of doubles for input
		RIntegerVector cIntVec;					// Temporary vectors of integers for input
		RComplexVector cCpxVec;					// Temporary vectors of complex values for input
		RStringVector cStrVec;					// Temporary vectors of strings for input
		RBoolVector cLglVec;					// Temporary vectors of boolean for input
		RFunction cFunc;						// Temporary function for input
		REnvironment cEnv;						// Temporary environment for input
		RList cList;							// Temporary list for input
		RObject cNull;							// Null object
		SEXP pCur;
		pCur = R_NilValue;
		try
		{
			if(TYPEOF(pRObIn) != LISTSXP)
			{
				StdExtensions::RaiseException("Input parameters are not of the correct list type", __LINE__, "RInterface::FillFromExternalParameters(SEXP)", __FILE__, StdExtensions::e_DomainError);
			}
			else
			{	// Ensure that the parameters input is a LISP-style list
				for(pCur = CDR(pRObIn); pCur != R_NilValue; pCur = CDR(pCur))
				{	// Iterate over the pairlist - importing each element
					switch(CalculateObjectType(CAR(pCur)))
					{
					case e_NULL:		// Input element is a null object
						if(PRINTNAME(TAG(pCur)) != NULL)
						{
							AddElementToBack(cNull, CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(cNull);
						}
						break;
					case e_LIST:		// Input element is a list
						cList.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cList), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cList));
						}
						break;
					case e_INTVEC:		// Input element is an integer vector
						cIntVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cIntVec));
						}
						break;
					case e_DBLVEC:		// Input element is a double vector
						cDblVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cDblVec));
						}
						break;
					case e_CPXVEC:		// Input element is a complex vector
						cCpxVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cCpxVec));
						}
						break;
					case e_STRVEC:		// Input element is a string vector
						cStrVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cStrVec));
						}
						break;
					case e_BOOLVEC:		// Input element is a boolean vector
						cLglVec.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cLglVec));
						}
						break;
					case e_FUNC:		// Input element is a function
						cFunc.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cFunc));
						}
						break;
					case e_ENV:			// Input element is an environment
						cEnv.ImportFromSEXP(CAR(pCur));
						if(PRINTNAME(TAG(pCur)) != R_NilValue)
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv), CHAR(PRINTNAME(TAG(pCur))));
						}
						else
						{
							AddElementToBack(dynamic_cast<RObject &>(cEnv));
						}
						break;
					default:			// Input element is an unsupported type
						StdExtensions::RaiseException("R object is an unsupported type", __LINE__, "RInterface::RList::FillFromExternalParameters(SEXP)", __FILE__, StdExtensions::e_DomainError);
						break;
					}
				}
			}
		}
		catch(StdExtensions::Exception cXa)
		{
			throw cXa;
		}
		catch(std::bad_cast)
		{
			StdExtensions::RaiseException("error thrown during casting process", __LINE__, "RInterface::RList::FillFromExternalParameters(SEXP)", __FILE__, StdExtensions::e_CastError);
		}
		catch(...)
		{
			StdExtensions::RaiseException("unknown error thrown during import", __LINE__, "RInterface::RList::FillFromExternalParameters(SEXP)", __FILE__, StdExtensions::e_UnknownError);
		}
	}

	// Export the list as an R SEXP object
	SEXP RList::ExportAsSEXP() const throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;				// Stack for holding protected SEXP objects
		SEXP pOut, pNamesOut, pDimsOut, pCurOut, pElementNames;
		size_t ulIter, ulLengthOut, ulEleNameLength;
		RList::RListLinker *pIter;
		char *pEleName;
		pOut = pNamesOut = pDimsOut = pCurOut = pElementNames = R_NilValue;
		ulIter = ulLengthOut = ulEleNameLength = 0;
		pIter = NULL;
		pEleName = NULL;
		if(pStart != NULL)
		{	// If the list has elements then create the relevant SEXP objects
			try
			{	// Calculate the length of the list
				ulLengthOut = Length();
				// Export the dimension names (if there are any)
				cProtStack.Protect(pNamesOut = ExportDimensionNamesAsSEXP());
				// Export the dimension extents (if there are any)
				cProtStack.Protect(pDimsOut = ExportDimensionExtentsAsSEXP());
				// Allocate a new list large enough to hold the list contents
				cProtStack.Protect(pOut = Rf_allocVector(VECSXP, int(ulLengthOut)));
				// Allocate a new vector of strings large enough to hold the element names
				cProtStack.Protect(pElementNames = Rf_allocVector(STRSXP, int(ulLengthOut)));
				// Iterate over the list, copying the elements to the output object
				pIter = pStart;
				for(ulIter = 0; ulIter < ulLengthOut; ++ulIter, pIter = pIter->pAfter)
				{
					cProtStack.Protect(pCurOut = pIter->pObject->ExportAsSEXP());
					SET_VECTOR_ELT(pOut, int(ulIter), pCurOut);
					if(!(pIter->GetTitle().IsMissingValue()))
					{	// If the element has a title then store it
						ulEleNameLength = pIter->GetTitle().Length();
						pEleName = StdExtensions::AllocateMemory<char>(ulEleNameLength + 1);
						try
						{	// Retrive the element name
							pIter->GetTitle(pEleName, ulEleNameLength + 1);
							// Set the string vector
							SET_STRING_ELT(pElementNames, int(ulIter), Rf_mkChar(pEleName));
						}
						catch(...)
						{	// Delete the name array if error is encountered
							delete [] pEleName;
							pEleName = NULL;
							throw;
						}
						delete [] pEleName;
						pEleName = NULL;
					}
					else
					{	// Element does not have a title
						SET_STRING_ELT(pElementNames, int(ulIter), NA_STRING);
					}
					cProtStack.Unprotect(1);
				}
				// Set the element titles
				Rf_setAttrib(pOut, R_NamesSymbol, pElementNames);
				if(pDimsOut != R_NilValue)
				{	// If the dimension extents exist then set the relevant attribute
					Rf_setAttrib(pOut, R_DimSymbol, pDimsOut);
				}
				if(pNamesOut != R_NilValue)
				{	// If dimension names exist then set the relevant attribute
					Rf_setAttrib(pOut, R_DimNamesSymbol, pNamesOut);
				}
			}
			catch(StdExtensions::Exception)
			{	// Standard exception raised
				cProtStack.UnprotectAll();		// Unprotect all protected SEXP objects
				throw;
			}
			catch(...)
			{	// Non-standard exception raised
				cProtStack.UnprotectAll();		// Unprotect all protected SEXP objects
				StdExtensions::RaiseException("unknown error raised during export of list object", __LINE__, "RInterface::RList::ExportAsSEXP()", __FILE__, StdExtensions::e_UnknownError);
			}
		}
		// Unprotect any protected tempory SEXP objects
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Function to return the length of the list
	size_t RList::Length() const throw()
	{
		RList::RListLinker *pIter;
		size_t ulTotLength = 0;
		pIter = pStart;
		// Otherwise iterate through the elements in the list - counting each one
		for(ulTotLength = 0; pIter != NULL; pIter = pIter->pAfter)
		{
			++ulTotLength;
		}
		return ulTotLength;
	}

	// Create an iterator at the start of the list
	RList::RListIterator RList::CreateNewIteratorFront() const throw(StdExtensions::Exception)
	{
		RList::RListIterator cOut;					// Create an iterator
		cOut.pCurElement = pStart;					// Set the current object that the iterator points to
		cOut.pParent = const_cast<RList *>(this);	// Set the list object that contains the current object that the iterator points to
		AddIteratorToCache(&cOut);					// Add the iterator to the cache
		return cOut;
	}

	// Create an iterator at the end of the list
	RList::RListIterator RList::CreateNewIteratorBack() const throw(StdExtensions::Exception)
	{
		RList::RListIterator cOut;					// Create an iterator
		cOut.pCurElement = pFinish;					// Set the current object that the iterator points to
		cOut.pParent = const_cast<RList *>(this);	// Set the list object that contains the current object that the iterator points to
		AddIteratorToCache(&cOut);					// Add the iterator to the cache
		return cOut;
	}

	// Retrieve an element from the list
	RObject& RList::GetElement(int iIter) const throw(StdExtensions::Exception)
	{
		int iGo = 0;
		RList::RListLinker *pIter;
		pIter = NULL;
		if(pStart == NULL)
		{	// List is empty - throw exception
			StdExtensions::RaiseException("no elements contained in the list", __LINE__, "RInterface::RList::GetElement(int)", __FILE__, StdExtensions::e_RangeError);
		}
		else
		{
			if(iIter < 0)	// Negative element selected - iterate backwards
			{	// Iterate over the list until the correct element is found
				for(pIter = pFinish; iGo < abs(iIter); ++iGo)
				{
					pIter = pIter->pBefore == NULL ? pFinish : pIter->pBefore;
				}
			}
			else			// Positive element selected - iterate forwards
			{	// Iterate over the list until the correct element is found
				for(pIter = pStart; iGo < iIter; ++iGo)
				{
					pIter = pIter->pAfter == NULL ? pStart : pIter->pAfter;
				}
			}
		}
		// Return the object pointed to by the iterator
		return const_cast<RObject &>(*(pIter->pObject));
	}

	// Retrieve an element from the list
	RObject& RList::GetElement(const int *pIter) const throw(StdExtensions::Exception)
	{
		return GetElement(ConvertDimLocToIter(pIter));
	}

	// Retrieve an element from the list
	RObject& RList::GetElement(const StdExtensions::ExtString &cTitle) const throw(StdExtensions::Exception)
	{
		RList::RListLinker *pIter;
		RObject *pOut;
		pOut = NULL;
		// Search through the list for an element with a title equal to cTitle
		for(pIter = pStart; pIter != NULL && pOut == NULL; pIter = pIter->pAfter)
		{
			if(cTitle.Equal(pIter->GetTitle()))
			{
				pOut = pIter->pObject;
			}
		}
		if(pOut == NULL)
		{
			StdExtensions::RaiseException("unable to find element with requested title in the target list", __LINE__, "RInterface::RList::GetElement(const StdExtensions::ExtString &)", __FILE__, StdExtensions::e_RangeError);
		}
		return const_cast<RObject &>(*pOut);
	}

	// ----------------------------------
	//  METHODS FOR THE R FUNCTION CLASS
	// ----------------------------------

	// Null initialisation function
	void RFunction::Initialise() throw()
	{
		if(pFunction != R_NilValue)
		{
			UNPROTECT_PTR(pFunction);
		}
		pFunction = R_NilValue;
	}

	// Initialise a function object from another function object
	void RFunction::Initialise(const RFunction &cObIn) throw()
	{
		Initialise();
		if(cObIn.pFunction != R_NilValue)
		{
			PROTECT(pFunction = cObIn.pFunction);
		}
	}

	// Call the function using the list as parameters
	SEXP RFunction::CallFunction(const RList &cObIn) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;				// Stack for holding protected SEXP objects
		SEXP pS, pT, pTemp, pOut, pNames;
		int iLength, iIter;
		pS = pT = pTemp = pOut = pNames = R_NilValue;
		iLength = iIter = 0;
		try
		{	// Calculate the length of the input list
			iLength = int(cObIn.Length());
			// Calculate temporary lists to create input vectors
			cProtStack.Protect(pTemp = cObIn.ExportAsSEXP());
			// Get the names attribute of the input list
			pNames = Rf_getAttrib(pTemp, R_NamesSymbol);
			cProtStack.Protect(pS = pT = Rf_allocList(iLength + 1));
			SET_TYPEOF(pS, LANGSXP);
			SETCAR(pT, pFunction);
			// Iterate through the input list and set the language parameters
			for(pT = CDR(pT); iIter < iLength; ++iIter, pT = CDR(pT))
			{
				SETCAR(pT, VECTOR_ELT(pTemp, iIter));
				SET_TAG(pT, Rf_install(CHAR(STRING_ELT(pNames, iIter))));
			}
			cProtStack.Protect(pOut = Rf_eval(pS, R_GlobalEnv));
		}
		catch(StdExtensions::Exception)
		{	// Standard exception thrown
			cProtStack.UnprotectAll();
			throw;
		}
		catch(...)
		{	// Non-standard exception thrown
			cProtStack.UnprotectAll();
			StdExtensions::RaiseException("unknown exception thrown during function call", __LINE__, "RInterface::RFunction::CallFunction(SEXP)", __FILE__, StdExtensions::e_UnknownError);
		}
		cProtStack.UnprotectAll();
		return pOut;
	}

	// Import information from an R SEXP object
	void RFunction::ImportFromSEXP(SEXP pRObIn) throw()
	{
		Initialise();
		if(CalculateObjectType(pRObIn) == e_FUNC)
		{
			if(pRObIn != R_NilValue)
			{
				PROTECT(pFunction = pRObIn);
			}
		}
		else
		{
			StdExtensions::RaiseException("input SEXP object is not a function", __LINE__, "StdExtensions::RFunction::Initialise(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
	}

	// Export the function as an R SEXP object
	SEXP RFunction::ExportAsSEXP() const throw()
	{
		return pFunction;
	}

	// Call an R function
	SEXP CallRFunction(const char *pFunc, const RList &cParams) throw(StdExtensions::Exception)
	{
		RProtectedStack cProtStack;				// Stack for holding protected SEXP objects
		SEXP pS, pT, pTemp, pOut, pNames;
		int iLength, iIter;
		pS = pT = pTemp = pOut = pNames = R_NilValue;
		iLength = iIter = 0;
		try
		{	// Calculate the length of the input list
			iLength = int(cParams.Length());
			// Calculate temporary lists to create input vectors
			cProtStack.Protect(pTemp = cParams.ExportAsSEXP());
			// Get the names attribute of the input list
			pNames = Rf_getAttrib(pTemp, R_NamesSymbol);
			cProtStack.Protect(pS = pT = Rf_allocList(iLength + 1));
			SET_TYPEOF(pS, LANGSXP);
			SETCAR(pT, Rf_install(pFunc));
			// Iterate through the input list and set the language parameters
			for(pT = CDR(pT); iIter < iLength; ++iIter, pT = CDR(pT))
			{
				SETCAR(pT, VECTOR_ELT(pTemp, iIter));
				SET_TAG(pT, Rf_install(CHAR(STRING_ELT(pNames, iIter))));
			}
			cProtStack.Protect(pOut = Rf_eval(pS, R_GlobalEnv));
		}
		catch(StdExtensions::Exception)
		{	// Standard exception thrown
			cProtStack.UnprotectAll();
			throw;
		}
		catch(...)
		{	// Non-standard exception thrown
			cProtStack.UnprotectAll();
			StdExtensions::RaiseException("unknown exception thrown during function call", __LINE__, "RInterface::CallRFunction(const char *, const RInterface::RList &)", __FILE__, StdExtensions::e_UnknownError);
		}
		cProtStack.UnprotectAll();
		return pOut;
	}
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
