// SpectralSynthesis.cpp : Defines the entry point for the DLL application (in Windows).

#include "stdafx.h"
#include "SpectralSynthesis.h"

//If the file is being compiled with a Visual C++ compiler then set it to be included once in the program only
//and to temporarily disable exception specification warnings generated as a result of non-standard behaviour
#ifdef _MSC_VER
	#pragma once
	#pragma warning( disable : 4290 )

	// Specify DllMain function
	BOOL APIENTRY DllMain( HANDLE hModule, 
	                       DWORD  ul_reason_for_call, 
	                       LPVOID lpReserved
						 )
	{
	    return TRUE;
	}

#endif

// Function to generate landscapes according to fractional Brownian motion
// this funciton uses an updated version of the algorithm published by
// Keitt (2000) allowing for conjugate symmetry conditions to be satisfied
// in all dimensions according to Peitgen and Saupe (1988)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		Hursts		-	Double vector describing the Hurst exponent in each dimension
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP SpectralSynthesis(SEXP pArgs)
#else
	extern "C" SEXP SpectralSynthesis(SEXP pArgs)
#endif
{	// R's random number generator
	RInterface::RRandom cRand;
	// Vectors to hold parameter inputs and output
	RInterface::RIntegerVector cDimExtents;
	RInterface::RDoubleVector cHursts, cOut;
	SEXP pOut = R_NilValue;
	// Parameters to hold derived input information
	int *pDimExtents;
	double *pHursts;
	size_t ulDimNumIn, ulIter;
	pDimExtents = NULL;
	pHursts = NULL;
	ulDimNumIn = ulIter = 0;
	try
	{	// Import information from the inputs
		cDimExtents.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "DimExtents"));
		cHursts.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "Hursts"));
		ulDimNumIn = cDimExtents.Length() > cHursts.Length() ?
			cDimExtents.Length() : cHursts.Length();	// Calculate the number of dimensions
		if(ulDimNumIn > 0)
		{	// Allocate memory for inputs
			pDimExtents = StdExtensions::AllocateMemory<int>(ulDimNumIn);
			pHursts = StdExtensions::AllocateMemory<double>(ulDimNumIn);
			// Copy the information across to the newly allocated arrays
			for(ulIter = 0; ulIter < ulDimNumIn; ++ulIter)
			{	// Set the dimension extents and hurst exponents
				pDimExtents[ulIter] = cDimExtents.GetElement(int(ulIter)).GetAsInteger();
				pHursts[ulIter] = cHursts.GetElement(int(ulIter)).GetAsDouble();
			}
			// Conduct spectral synthesis
			cOut.SpectralSynthesis(ulDimNumIn, pDimExtents, pHursts, dynamic_cast<StdExtensions::Random *>(&cRand));
			// Export the output to a SEXP object
			pOut = cOut.ExportAsSEXP();
		}
	}
	catch(StdExtensions::Exception cXa)
	{	// Garbage collection - delete allocated arrays
		if(pDimExtents != NULL)
		{
			delete [] pDimExtents;
		}
		if(pHursts != NULL)
		{
			delete [] pHursts;
		}
		pDimExtents = NULL;
		pHursts = NULL;
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{	// Garbage collection - delete allocated arrays
		if(pDimExtents != NULL)
		{
			delete [] pDimExtents;
		}
		if(pHursts != NULL)
		{
			delete [] pHursts;
		}
		pDimExtents = NULL;
		pHursts = NULL;
		// Write error if casting exception is thrown
		Rf_error("casting error thrown during random number generator conversion");
	}
	catch(...)
	{	// Garbage collection - delete allocated arrays
		if(pDimExtents != NULL)
		{
			delete [] pDimExtents;
		}
		if(pHursts != NULL)
		{
			delete [] pHursts;
		}
		pDimExtents = NULL;
		pHursts = NULL;
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	// Garbage collection - delete allocated arrays
	if(pDimExtents != NULL)
	{
		delete [] pDimExtents;
	}
	if(pHursts != NULL)
	{
		delete [] pHursts;
	}
	pDimExtents = NULL;
	pHursts = NULL;
	return pOut;
}

// Function to generate landscapes according to hierarchical curdling algorithms
// of O'Neill, Gardner and Turner (1992) and Lavorel, Gardner and O'Neill (1993)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		HierBoxes	-	A matrix describing the size of the tiles at each hierarchical
//						subdivision.  The first dimension of the matrix is the hierarchy
//						of the tile and the second dimension of matrix descirbes the
//						size of the tile in each dimension of the output
//		HierProp	-	Integer vector describing the number of tiles to keep
//						at each hierarchical sweep
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP HierarchicalCurdling(SEXP pArgs)
#else
	extern "C" SEXP HierarchicalCurdling(SEXP pArgs)
#endif
{	// R's random number generator
	RInterface::RRandom cRand;
	// Vectors to hold parameter inputs and output
	RInterface::RIntegerVector cDimExtents, cHierBoxes, cOut;
	RInterface::RIntegerVector cHierProp;
	SEXP pOut = R_NilValue;				// Output value
	int *pHierProp;						// Array to store the hierarchical proportions
	int **pHierBoxes, *pDimExtents;		// Arrays to store the size of the dimensional tiles and the dimension extents
	int iDimNum, iHierNum;				// The number of dimensions and hierarchies
	int iDimIter, iHierIter;
	int pCoords[2];
	pHierProp = NULL;
	iDimNum = iHierNum = iDimIter = iHierIter = 0;
	pCoords[0] = pCoords[1] = 0;
	pHierBoxes = NULL;
	pDimExtents = NULL;
	try
	{	// Import information from the inputs
		cDimExtents.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "DimExtents"));
		iDimNum = int(cDimExtents.Length());
		cHierBoxes.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "HierBoxes"));
		cHierProp.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "HierProp"));
		iHierNum = int(cHierBoxes.GetDimensionExtent(0) > cHierProp.Length() ?
			cHierBoxes.GetDimensionExtent(0) : cHierProp.Length());
		// If the hierachical boxes input has two dimensions then ensure that
		// the number of dimensions is at the highest number
		if(cHierBoxes.GetNumberDimensions() == 2)
		{
			iDimNum = int(cHierBoxes.GetDimensionExtent(1)) > iDimNum ? int(cHierBoxes.GetDimensionExtent(1)) : iDimNum;
		}
		// Ensure that correct values exist for the number of dimensions and the number of hierarchies
		if(iHierNum <= 0)
		{	// Raise exception if there are an invalid number of hierarchies
			StdExtensions::RaiseException("invalid number of hierarchies", __LINE__, "HierarchicalCurdling(SEXP)", __FILE__, StdExtensions::e_DomainError);
		}
		else if(iDimNum > 0)
		{	// Allocate memory for inputs
			pDimExtents = StdExtensions::AllocateMemory<int>(size_t(iDimNum));
			pHierBoxes = StdExtensions::AllocateMemory<int *>(size_t(iHierNum));
			pHierProp = StdExtensions::AllocateMemory<int>(size_t(iHierNum));
			// Fill the dimension extent input vector
			for(iDimIter = 0; iDimIter < iDimNum; ++iDimIter)
			{
				pDimExtents[size_t(iDimIter)] = cDimExtents.GetElement(iDimIter).GetAsInteger();
			}
			// Fill the hierarchical information
			for(iHierIter = 0; iHierIter < iHierNum; ++iHierIter)
			{	// Set the proportion of selected cells at the current hierarchy
				pHierProp[size_t(iHierIter)] = cHierProp.GetElement(iHierIter).GetAsInteger();
				// Initialise hierarchical boxes to null elements
				pHierBoxes[size_t(iHierIter)] = NULL;
				pHierBoxes[size_t(iHierIter)] = StdExtensions::AllocateMemory<int>(size_t(iDimNum));
				if(cHierBoxes.GetNumberDimensions() == 1)
				{	// Hierarchical box is cubic at each level
					for(iDimIter = 0; iDimIter < iDimNum; ++iDimIter)
					{
						pHierBoxes[size_t(iHierIter)][size_t(iDimIter)] = cHierBoxes.GetElement(iHierIter).GetAsInteger();
					}
				}
				else if(cHierBoxes.GetNumberDimensions() == 2)
				{	// Hierarchical box is rectangular at each level
					pCoords[0] = iHierIter;
					for(iDimIter = 0; iDimIter < iDimNum; ++iDimIter)
					{
						pCoords[1] = iDimIter;
						pHierBoxes[size_t(iHierIter)][size_t(iDimIter)] = cHierBoxes.GetElement(pCoords).GetAsInteger();
					}
				}
				else
				{	// Hierarchical box matrix has an invalid number of dimensions
					StdExtensions::RaiseException("hierachical tile size input has an invalid dimension structure", __LINE__, "HierarchicalCurdling(SEXP)", __FILE__, StdExtensions::e_DomainError);
				}
			}
			--iHierIter;
			// Create the hierarchical curdled landscape
			cOut.HierarchicalCurdling(size_t(iDimNum), size_t(iHierNum), pDimExtents, pHierBoxes, pHierProp, dynamic_cast<StdExtensions::Random *>(&cRand));
			// Create the output object
			pOut = cOut.ExportAsSEXP();
			// Delete all the temporary arrays
			for(; iHierIter >= 0; --iHierIter)
			{
				delete [] pHierBoxes[size_t(iHierIter)];
				pHierBoxes[size_t(iHierIter)] = NULL;
			}
			delete [] pHierBoxes;
			delete [] pHierProp;
			delete [] pDimExtents;
			pHierBoxes = NULL;
			pHierProp = NULL;
			pDimExtents = NULL;
		}
	}
	catch(StdExtensions::Exception cXa)
	{	// Garbage collection
		if(pHierBoxes != NULL)
		{	// Delete the hierarchical envelope information
			for(; iHierIter >= 0; --iHierIter)
			{
				if(pHierBoxes[size_t(iHierIter)] != NULL)
				{
					delete [] pHierBoxes[size_t(iHierIter)];
					pHierBoxes[size_t(iHierIter)] = NULL;
				}
			}
			delete [] pHierBoxes;
			pHierBoxes = NULL;
		}
		if(pHierProp != NULL)
		{	// Delete the hierarchical proportions
			delete [] pHierProp;
			pHierProp = NULL;
		}
		if(pDimExtents != NULL)
		{	// Delete the dimension extents
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{	// Garbage collection
		if(pHierBoxes != NULL)
		{	// Delete the hierarchical envelope information
			for(; iHierIter >= 0; --iHierIter)
			{
				if(pHierBoxes[size_t(iHierIter)] != NULL)
				{
					delete [] pHierBoxes[size_t(iHierIter)];
					pHierBoxes[size_t(iHierIter)] = NULL;
				}
			}
			delete [] pHierBoxes;
			pHierBoxes = NULL;
		}
		if(pHierProp != NULL)
		{	// Delete the hierarchical proportions
			delete [] pHierProp;
			pHierProp = NULL;
		}
		if(pDimExtents != NULL)
		{	// Delete the dimension extents
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		// Write error if casting exception is thrown
		Rf_error("casting error thrown during random number generator conversion");
	}
	catch(...)
	{	// Garbage collection
		if(pHierBoxes != NULL)
		{	// Delete the hierarchical envelope information
			for(; iHierIter >= 0; --iHierIter)
			{
				if(pHierBoxes[size_t(iHierIter)] != NULL)
				{
					delete [] pHierBoxes[size_t(iHierIter)];
					pHierBoxes[size_t(iHierIter)] = NULL;
				}
			}
			delete [] pHierBoxes;
			pHierBoxes = NULL;
		}
		if(pHierProp != NULL)
		{	// Delete the hierarchical proportions
			delete [] pHierProp;
			pHierProp = NULL;
		}
		if(pDimExtents != NULL)
		{	// Delete the dimension extents
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

// Function to generate a simple percolation map according to the methods described
// in Gardner et al. (1987)
//
//		DimExtents	-	Integer vector describing the length of each dimension
//		Props		-	Integer vector of proportions of the land to designate to
//						each habitat type
//
#ifdef _MSC_VER
	extern "C" _declspec(dllexport) SEXP PercolationMap(SEXP pArgs)
#else
	extern "C" SEXP PercolationMap(SEXP pArgs)
#endif
{	// R's random number generator
	RInterface::RRandom cRand;
	// Vectors to hold parameter inputs and output
	RInterface::RIntegerVector cDimExtents, cProps, cOut;
	SEXP pOut = R_NilValue;				// Output value
	int *pDimExtents, *pProps;			// Arrays to store inputs
	size_t ulDimNum, ulNumHabs, ulIter;
	pDimExtents = pProps = NULL;
	ulDimNum = ulIter = 0;
	try
	{	// Import information from the inputs
		cDimExtents.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "DimExtents"));
		ulDimNum = cDimExtents.Length();		// Set the number of dimensions
		cProps.ImportFromSEXP(RInterface::GetParameterItem(pArgs, "Props"));
		ulNumHabs = cProps.Length();			// Set the number of habitat types
		pDimExtents = StdExtensions::AllocateMemory<int>(ulDimNum);
		pProps = StdExtensions::AllocateMemory<int>(ulNumHabs);
		// Fill in the dimension extents
		for(ulIter = 0; ulIter < ulDimNum; ++ulIter)
		{
			pDimExtents[ulIter] = cDimExtents.GetElement(int(ulIter)).GetAsInteger();
		}
		// Fill in the habitat proportion vector
		for(ulIter = 0; ulIter < ulNumHabs; ++ulIter)
		{
			pProps[ulIter] = cProps.GetElement(int(ulIter)).GetAsInteger();
		}
		// Calculate the percolation map
		cOut.PercolationMap(ulDimNum, ulNumHabs, pDimExtents, pProps, dynamic_cast<StdExtensions::Random *>(&cRand));
		// Export the output
		pOut = cOut.ExportAsSEXP();
		// Garbage collection
		delete [] pDimExtents;
		delete [] pProps;
		pDimExtents = pProps = NULL;
	}
	catch(StdExtensions::Exception cXa)
	{
		if(pDimExtents != NULL)
		{	// Delete the extents array
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		if(pProps != NULL)
		{	// Delete the proportions array
			delete [] pProps;
			pProps = NULL;
		}
		// Write error if a standard exception is thrown
		RInterface::WriteRException(cXa);
	}
	catch(std::bad_cast)
	{
		if(pDimExtents != NULL)
		{	// Delete the extents array
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		if(pProps != NULL)
		{	// Delete the proportions array
			delete [] pProps;
			pProps = NULL;
		}
		// Write error if casting exception is thrown
		Rf_error("casting error thrown during random number generator conversion");
	}
	catch(...)
	{
		if(pDimExtents != NULL)
		{	// Delete the extents array
			delete [] pDimExtents;
			pDimExtents = NULL;
		}
		if(pProps != NULL)
		{	// Delete the proportions array
			delete [] pProps;
			pProps = NULL;
		}
		// Write error if a non-standard exception is thrown
		Rf_error("unknown error");
	}
	return pOut;
}

//Return warnings to default upon compilation with Visual C++ compiler
#ifdef _MSC_VER
	#pragma warning( default : 4290 )
#endif
